package book;

import java.util.*;

/**
 * <h2>Error.</h2>
 *
 * <p>This class provides a way to store non-fatal errors. It contains the
 * exception that caused the error, a description of the error (that in
 * addition to the exception message describes where the exception was caught),
 * and the date at which the error occured.</p>
 *
 * @author Antoine Dutot
 * @since 20041107
 */
public class Error
{
// Attributes

	/**
	 * Exception that caused the error.
	 */
	public Throwable exception;

	/**
	 * Error description.
	 */
	public String error;

	/**
	 * Date of the error.
	 */
	public long date;

// Constructors

	/**
	 * New error.
	 * @param error Description of the error.
	 */
	public
	Error( String error )
	{
		this( error, null );
	}

	/**
	 * New error.
	 * @param error Description of the error.
	 * @param exception Exception that caused the error.
	 */
	public
	Error( String error, Throwable exception )
	{
		this.error     = error;
		this.exception = exception;
		this.date      =  System.currentTimeMillis();
	}

// Accessors

	/**
	 * Error description.
	 */
	public String
	getError()
	{
		return error;
	}

	/**
	 * Raised exception (if any) or null.
	 */
	public Throwable
	getException()
	{
		return exception;
	}

	/**
	 * Date of the error (milliseconds from epoch).
	 */
	public long
	getDate()
	{
		return date;
	}

// Util.

	public String
	toString()
	{
		StringBuffer buf = new StringBuffer();

		Date date = new Date( this.date );

		buf.append( "Error ("+date+"): '" );
		buf.append( error );
		buf.append( "'" );

		if( exception != null )
		{
			buf.append( " caused by exception '" + exception.getClass().getName() +"': "+ exception.getMessage() );
		}

		return buf.toString();
	}
}
