package book;

/**
 * <h2>Group of two values.</h2>
 *
 * <p>A Pair is most of the time used to return two values at once.</p>
 *
 * @author Antoine Dutot
 * @since 20010212
 * @version 0.1
 */
public class Pair<E,F>
	implements Cloneable
{
// Attributes

	/**
	 * First component.
	 */
	public E first;

	/**
	 * Second component.
	 */
	public F second;

// Constructors

	/**
	 * New empty pair.
	 */
	public
	Pair()
	{
		first  = null;
		second = null;
	}

	/**
	 * New [<code>first</code>,<code>second</code>] pair.
	 * @param first The first value of the pair.
	 * @param second The second value of the pair.
	 */
	public
	Pair( E first, F second )
	{
		this.first  = first;
		this.second = second;
	}

	/**
	 * New copy of <code>other</code>.
	 * @param other The pair to copy.
	 */
	public
	Pair( Pair<? extends E,? extends F> other )
	{
		first  = other.first;
		second = other.second;
	}

// Accessors

	/**
	 * First value of the pair.
	 * @return The first value.
	 */
	public E
	getFirst()
	{
		return first;
	}

	/**
	 * Second value of the pair.
	 * @return The second value.
	 */
	public F
	getSecond()
	{
		return second;
	}

// Commands

	/**
	 * Change the pair to [<code>first</code>,</code>second</code>].
	 * @param first The first value of the pair.
	 * @param second The second value of the pair.
	 */
	public void
	set( E first, F second )
	{
		this.first  = first;
		this.second = second;
	}

	/**
	 * Change the <code>first</code> value.
	 * @param first The first value of the pair.
	 */
	public void
	setFirst( E first )
	{
		this.first = first;
	}

	/**
	 * Change the <code>second</code> value.
	 * @param second The second value of the pair.
	 */
	public void
	setSecond( F second )
	{
		this.second = second;
	}

	/**
	 * Copy <code>other</code> into this.
	 * @param other The pair to copy.
	 */
	public void
	copy( Pair<? extends E,? extends F> other )
	{
		first  = other.first;
		second = other.second;
	}
}
