package book;

/**
 * <h2>A test facility class. This class is meant to be inherited and provides a
 * really small set of helper methods to make test programs.</h2>
 *
 * @author Antoine Dutot
 * @since 19990811
 * @deprecated Use unit test frameworks instread.
 */
public class TestFacility
{
// Attributes
	
	/**
	 * How many errors?.
	 */
	int error_count;

	/**
	 * Print a report of each error?.
	 */
	boolean show_errors;

// Constructors

	public
	TestFacility()
	{
		this ( false );
	}
	
	public
	TestFacility( boolean show_errors )
	{
		this.error_count = 0;
		this.show_errors = show_errors;
	}

// Commands

	/**
	 * Allow or disallow to report each error.
	 */
	public void
	set_show_errors( boolean on )
	{
		show_errors = on;
	}

	/**
	 * Increment error count if predicate is false.
	 */
	public void
	do_test( boolean predicate )
	{
		if( ! predicate )
		{
			++ error_count;

			if( show_errors )
			{
				Thread.dumpStack();
			}
		}
	}
	
	/**
	 * Increment error count if predicate is false and print message on stderr
	 * if message is not null.
	 */
	public void
	do_test( boolean predicate, String message )
	{
		if( ! predicate )
		{
			++ error_count;

			if( message != null )
			{
				System.err.print( "[error: " );
				System.err.print( message );
				System.err.print( "]\n" );
			}
		}
	}
	
	/**
	 * Echo a localized error count message to out.
	 */
	public void
	print_test_status()
	{
		if( error_count == 0 )
		{
		//	System.out.print( GREEN );
			System.out.print( "No errors!\n" );
		//	System.out.print( RESET );
		}
		else if( error_count == 1 )
		{
		//	System.err.print( RED );
			System.err.print( error_count );
			System.err.print( " error" );
		//	System.err.print( RESET );
			System.err.print( '\n' );
		}
		else
		{
		//	System.err.print( RED );
			System.err.print( error_count );
			System.err.print( " errors" );
		//	System.err.print( RESET );
			System.err.print( '\n' );
		}
	}

// Accessors
	
	/**
	 * Number of failed tests so far.
	 */
	public int
	get_test_error_count()
	{
		return error_count;
	}
	
	/**
	 * Suitable exit code for error count. Most systems only support a limited
	 * range of return values (usually in [0..255]. This method returns either
	 * success (all test passed) or failure (at least a test failed).
	 */
	public int
	get_test_exit_status()
	{
		if( error_count > 0 )
		     return 1;
		else return 0;
	}
}
