package book.geom;

/**
 * <h2>Fast maths.</h2>
 *
 * <p>This class contains tables and precomputed values to fasten some math
 * operations.</p>
 *
 * @author Antoine Dutot
 * @since 20030819
 */
public class FastMath
{
	public static final double COS[] = new double[360];
	public static final double SIN[] = new double[360];

	static
	{
		double t, p = Math.PI / 180.0;
		for( int i=0; i<360; ++i )
		{
			t = i * p;
			COS[i] = java.lang.Math.cos( t );
			SIN[i] = java.lang.Math.sin( t );
		}
	}

	/**
	 * Precomputed cosinus values for integer angles. Negative values and
	 * values larger than 359 or smaller than -359 are accepted.
	 * @param value The value we search the cosinus for.
	 * @return The cosinus of the <code>value</code>.
	 */
	public static double
	cos( int value )
	{
		double sign = 1;

		if( value < 0 )
		{
			value = -value;
			sign  = -1;
		}

		if( value > 360 )
		{
			value = value % 360;
		}

		assert value >= 0 && value < 360 : "value must be >= 0 and < 360";

		return( COS[value] * sign );
	}
	
	/**
	 * Like {@link #cos(int)} but returns a float.
	 * @param value The value we search the cosinus for.
	 * @return The cosinus of the given value.
	 */
	public static float
	cosf( int value )
	{
		return (float) cos( value );
	}

	/**
	 * Precomputed sinus values for integer angles. Negative values and values
	 * larger than 359 or smaller than -359 are accepted.
	 * @param value The value we search the sinus for.
	 * @return The sinus of the <code>value</code>.
	 */
	public static double
	sin( int value )
	{
		double sign = 1;

		if( value < 0 )
		{
			value = -value;
			sign  = -1;
		}

		if( value > 360 )
		{
			value = value % 360;
		}

		assert value >= 0 && value < 360 : "value must be >=0 and < 360";

		return( SIN[value] * sign );
	}
	
	/**
	 * Like {@link #sin(int)} but returns a float.
	 * @param value The value we search the sinus for.
	 * @return The sinus of the given value.
	 */
	public static float
	sinf( int value )
	{
		return (float) sin( value );
	}
}
