package book.geom;

/**
 * Like the Point3 and InterpolatedFloat classes.
 *
 * @author antoine
 * @since  7 avr. 2006
 */
public class
	InterpolatedPoint3
extends
	Point3
{
//	 Attributes

	private static final long serialVersionUID = -2389923090511259123L;

	/**
	 * Objective value.
	 */
	protected float destX, destY, destZ = 0;
	
	/**
	 * Delta multiplier. The delta is the difference between the current value and
	 * the objective value.
	 */
	protected float deltaMult = 0.05f;
	
	/**
	 * Interpolation limit.
	 */
	protected float limit;

// Constructors

	public
	InterpolatedPoint3( float initialX, float initialY, float initialZ, float deltaMult )
	{
		super( initialX, initialY, initialZ );

		this.destX     = initialX;
		this.destY     = initialY;
		this.destZ     = initialZ;
		this.deltaMult = deltaMult;
	}
	
// Accessors
	
	/**
	 * The interpolated value.
	 * @return The value.
	public float
	getValue()
	{
		return value;
	}
	*/
	
	/**
	 * The non interpolated value.
	 * @return The value.
	public float
	getDirectValue()
	{
		return destValue;
	}
	*/

	public void
	copy( Point3 other )
	{
		destX = other.x;
		destY = other.y;
		destZ = other.z;
		limit = (float) (Math.abs( destX - x ) * 0.001f );
}

	public void
	set( float x, float y, float z )
	{
		destX = x;
		destY = y;
		destZ = z;
		limit = (float) (Math.abs( destX - x ) * 0.001f );
	}

// Commands
	
	/**
	 * Effectively product the smooth scaling.
	 */
	public void
	energy()
	{
		float deltaX = ( destX - x ) * deltaMult; 
		float deltaY = ( destY - y ) * deltaMult; 
		float deltaZ = ( destZ - z ) * deltaMult; 
		
		if( Math.abs( deltaX ) > limit )
		     x += deltaX;
		else x = destX;
		
		if( Math.abs( deltaY ) > limit )
		     y += deltaY;
		else y = destY;
		
		if( Math.abs( deltaZ ) > limit )
		     z += deltaZ;
		else z = destZ;
	}
	
	public void
	moveTo( float x, float y, float z )
	{
		destX = x;
		destY = y;
		destZ = z;
		limit = (float) (Math.abs( destX - x ) * 0.001f );
	}
	
	public void
	move( float dx, float dy, float dz )
	{
		destX += dx;
		destY += dy;
		destZ += dz;
	}

	public void
	move( Point3 p )
	{
		destX += p.x;
		destY += p.y;
		destZ += p.z;
	}
	
	public void
	move( Vector3 d )
	{
		destX += d.data[0];
		destY += d.data[1];
		destZ += d.data[2];
	}

	public void
	moveX( float dx )
	{
		destX += dx;
	}
	
	public void
	moveY( float dy )
	{
		destY += dy;
	}
	
	public void
	moveZ( float dz )
	{
		destZ += dz;
	}

	public void
	scale( float sx, float sy, float sz )
	{
		destX *= sx;
		destY *= sy;
		destZ *= sz;
	}

	public void
	scale( Point3 s )
	{
		destX *= s.x;
		destY *= s.y;
		destZ *= s.z;
	}

	public void
	scale( Vector3 s )
	{
		destX *= s.data[0];
		destY *= s.data[1];
		destZ *= s.data[2];
	}

	public void
	setX( float x )
	{
		destX = x;
		limit = (float) (Math.abs( destX - x ) * 0.001f );
	}
	
	public void
	setY( float y )
	{
		destY = y;
		limit = (float) (Math.abs( destY - y ) * 0.001f );
	}
	
	public void
	setZ( float z )
	{
		destZ = z;
		limit = (float) (Math.abs( destZ - z ) * 0.001f );
	}

	public void
	swap( Point3 other )
	{
		float t;

		if( other != this )
		{
			t       = destX;
			destX   = other.x;
			other.x = t;
			
			t       = destY;
			destY   = other.y;
			other.y = t;
			
			t       = destZ;
			destZ   = other.z;
			other.z = t;
			
			limit = (float) (Math.abs( destX - x ) * 0.001f );
		}
	}
}