package book.geom;

/**
 * <h2>Rotation transformation.</h2>
 *
 * <p>A rotation is a  4 cell vector whose first component is the rotation
 * angle in degrees and the three last components indicate the rotation
 * axis.</p>
 *
 * @author Antoine Dutot
 * @since 20040516
 */
public class RotateTransform
	extends
		Vector4
	implements
		Transform
{
// Constructors

	private static final long serialVersionUID = -3893663453184186001L;

	/**
	 * Null rotation (0,0,0,1).
	 */
	public
	RotateTransform()
	{
		super( 0, 0, 0, 1 );
	}

	/**
	 * Rotate of <code>angle</code> (degrees) arround axis
	 * (<code>rx</code>,<code>ry</code>,<code>rz</code>).
	 */
	public
	RotateTransform( float angle, float rx, float ry, float rz )
	{
		super( angle, rx, ry, rz );
	}

// Accessors

	/**
	 * Rotation angle in degree.
	 */
	public float
	getAngle()
	{
		return data[0];
	}

	/**
	 * Axis X component.
	 */
	public float
	getX()
	{
		return data[1];
	}

	/**
	 * Axis Y component.
	 */
	public float
	getY()
	{
		return data[2];
	}

	/**
	 * Axis Z component.
	 */
	public float
	getZ()
	{
		return data[3];
	}
}
