package book.geom;

import java.util.*;

/**
 * <h2>Sequence of transformations.</h2>
 *
 * @author Antoine Dutot
 * @since 20040521
 */
public class TransformStack
{
// Attributes

	/**
	 * List of transformations, allocated only when needed.
	 */
	protected ArrayList<Transform> transforms = null;

// Constructors

	/**
	 * Empty transformation stack.
	 */
	public
	TransformStack()
	{
	}

	/**
	 * New transformation stack with an initial <code>transform</code>.
	 */
	public
	TransformStack( Transform transform )
	{
		add( transform );
	}

// Accessors

	/**
	 * Number of transforms.
	 * @see #getTransform(int)
	 */
	public int
	getCount()
	{
		if( transforms == null )
			return 0;

		return transforms.size();
	}

	/**
	 * I-th transform. A transform contains specific informations to specify
	 * how sub-sprites are transformed (rotated, translated, scaled, etc.).
	 * Transforms are ordered.
	 */
	public Transform
	get( int i )
	{
		if( transforms == null )
			throw new ArrayIndexOutOfBoundsException( "transformation stack is empty" );

		return transforms.get( i );
	}

// Commands

	/**
	 * remove all transformations.
	 */
	public void
	clear()
	{
		transforms.clear();
		transforms = null;
	}

	/**
	 * Append a <code>transform</code> to this sprite. Transformations are
	 * applied in order.
	 * @see #putTransform(int,Transform)
	 */
	public void
	add( Transform transform )
	{
		if( transforms == null )
			transforms = new ArrayList<Transform>();

		transforms.add( transform );
	}

	/**
	 * Insert a <code>transform</code> at position <code>i</code>.
	 * Transformations are applied in order.
	 * @see #addTransform(Transform)
	 */
	public void
	put( int i, Transform transform )
	{
		if( transforms == null )
			transforms = new ArrayList<Transform>();

		transforms.add( i, transform );
	}

	/**
	 * Remove the <code>i</code>-th transform.
	 */
	public void
	remove( int i )
	{
		if( transforms != null )
		{
			transforms.remove( i );

			if( transforms.size() <= 0 )
				transforms = null;
		}
	}
}
