package book.geom;

/**
 * <h2>Viewport specifications.</h2>
 *
 * @author Antoine Dutot
 * @since 20041014
 */
public class Viewport
{
// Attributes

	/**
	 * Viewport x, y, width and height in this order.
	 */
	protected int vp[] = new int[4];

// Constructors

	/**
	 * New viewport at position <code>(x,y)</code>, with size
	 * <code>(w,h)</code>.
	 * @param x Abscissa of the viewport in pixels.
	 * @param y Ordinate of the viewport in pixels.
	 * @param w Width of the viewport in pixels.
	 * @param h Height of the viewport in pixels.
	 */
	public
	Viewport( int x, int y, int w, int h )
	{
		set( x, y, w, h );
	}

	/**
	 * New copy of the <code>other</code> viewport.
	 * @param other Viewport to copy.
	 */
	public
	Viewport( Viewport other )
	{
		copy( other );
	}

// Accessors

	/**
	 * @return Abscissa of the viewport in pixels.
	 */
	public int
	getX()
	{
		return vp[0];
	}

	/**
	 * @return Ordinate of the viewport in pixels.
	 */
	public int
	getY()
	{
		return vp[1];
	}

	/**
	 * @return Width of the viewport in pixels.
	 */
	public int
	getWidth()
	{
		return vp[2];
	}

	/**
	 * @return Height of the viewport in pixels.
	 */
	public int
	getHeight()
	{
		return vp[3];
	}

	/**
	 * Viewport as an array of four integers in order: x, y, width and height.
	 */
	public int[]
	getViewportArray()
	{
		return vp;
	}

// Commands

	/**
	 * Copy the <code>other</code> viewport.
	 * @param other The viewport to copy.
	 */
	public void
	copy( Viewport other )
	{
		set( other.vp[0], other.vp[1], other.vp[2], other.vp[3] );
	}

	/**
	 * Set the viewport at position <code>(x,y)</code>, with size
	 * <code>(w,h)</code>.
	 * @param x Abscissa of the viewport in pixels.
	 * @param y Ordinate of the viewport in pixels.
	 * @param w Width of the viewport in pixels.
	 * @param h Height of the viewport in pixels.
	 */
	public void
	set( int x, int y, int w, int h )
	{
		vp[0] = x;
		vp[1] = y;
		vp[2] = w;
		vp[3] = h;
	}

	/**
	 * Set the position of the viewport at <code>(x,y)</code>.
	 * @param x Abscissa of the viewport in pixels.
	 * @param y Ordinate of the viewport in pixels.
	 */
	public void
	setPosition( int x, int y )
	{
		vp[0] = x;
		vp[1] = y;
	}

	/**
	 * Set the size of the viewport <code>(w,h)</code> in pixels.
	 * @param w Width of the viewport in pixels.
	 * @param h Height of the viewport in pixels.
	 */
	public void
	setSize( int w, int h )
	{
		vp[2] = w;
		vp[3] = h;
	}
}
