package book.set;

/**
 * Iterator for the doubly linked list class.
 *
 * @author Antoine Dutot
 * @since 19980707
 * @version 0.1
 */
public class BilinkedListIterator
	implements Iterator, Cloneable
{
// Attributes
	
	/**
	 * Current position in the list or null.
	 */
	Bilinkable node;
	
// Constructors

	/**
	 * New iterator pointing on nothing.
	 */
	public
	BilinkedListIterator()
	{
		node = null;
	}

	/**
	 * New iterator pointing on node.
	 */
	public
	BilinkedListIterator( Bilinkable node )
	{
		this.node = node;
	}
	
	/**
	 * New iterator pointing on first node of list.
	 */
	public
	BilinkedListIterator( BilinkedList list )
	{
		this.node = list.ring.next;
	}
	
	/**
	 * New copy of other.
	 */
	public
	BilinkedListIterator( BilinkedListIterator other )
	{
		this.node = other.node;
	}

// Accessors

	/**
	 * No successor on wich to iterate?
	 *
	 * For bilinked iterators there is never a end, lists are loops. You must
	 * use the #equals() method and compare the iterator with a specific
	 * iterator returned by the list that indicates the end.
	 */
	public boolean
	is_end()
	{
		return false;
	}

	/**
	 * Pointing on the same element as other?
	 */
	public boolean
	equals( BilinkedListIterator other )
	{
		return( node == other.node );
	}

	/**
	 * Pointing on the same element as other?
	 */
	public boolean
	equals( Iterator other )
	{
		if( other instanceof BilinkedListIterator )
		{
			return( ((BilinkedListIterator)other).node == node );
		}

		return false;
	}

	/**
	 * Pointed at list node or null.
	 */
	public Bilinkable
	get_node()
	{
		return node;
	}

	/**
	 * Value pointed at or null.
	 */
	public Object
	get_value()
	{
		if( node instanceof ObjectBilinkable )
		{
			return((ObjectBilinkable)node).value;
		}

		return null;
	}
	
// Commands

	/**
	 * Copy the value of other(but do not point on the same node as other).
	 */
	public void
	copy( BilinkedListIterator other )
	{
		node = other.node;
	}

	/**
	 * Change the pointed at node.
	 */
	public void
	set_node( Bilinkable node )
	{
		this.node = node;
	}

	/**
	 * Go to next element in the list.
	 */
	public void
	next()
	{
		node = node.next;
	}
	
	/**
	 * Go to previous element in the list.
	 */
	public void
	previous()
	{
		node = node.prev;
	}
}

//  vim:ts=4:ai:
