package book.set;

/**
 * Iterator for the Deque.
 *
 * @author Antoine Dutot.
 * @since 19991025
 * @version 0.1
 */
public class DequeIterator
	implements Iterator
{
// Attributes

	/**
	 * Pointed at Deque.
	 */
	protected Deque deck;
	
	/**
	 * Position.
	 */
	protected int cursor;

// Constructors

	public
	DequeIterator()
	{
		deck = null;
		cursor = 0;
	}

	/**
	 * Iterator pointing at current in deque.
	 */
	public
	DequeIterator( Deque deque, int current )
	{
		deck = deque;
		cursor = current;
	}
	
	/**
	 * Copy of other.
	 */
	public
	DequeIterator( DequeIterator other )
	{
		deck = other.deck;
		cursor = other.cursor;
	}
	
// Predicates

	/**
	 * Point on a valid cell in the deque ?
	 *
	 * The iterator can be used to access the value it points to. Else it
	 * points out of the deque addressing space.
	 */
	public boolean
	is_valid()
	{
		if( deck.beg <= deck.pte )
		{
			return( cursor >= deck.beg && cursor < deck.pte );
		}
		else
		{
			return( ( cursor >= 0        && cursor < deck.pte ) 
			    ||   ( cursor >= deck.beg && cursor < deck.cap ) );
		}
	}

	/**
	 * The iterator points past-the-last usable cell ?
	 */
	public boolean
	is_end()
	{
		return( cursor == deck.pte );
	}

	/**
	 * The iterator points on the first usable cell ?
	 */
	public boolean
	is_begin()
	{
		return( cursor == deck.beg );
	}

	/**
	 * Equal to other ?
	 */
	public boolean
	equals( DequeIterator other )
	{
		return( deck == other.deck
		    &&   cursor == other.cursor );
	}

	/**
	 * Equal to other ?
	 */
	public boolean
	equals( Iterator other )
	{
		if( other instanceof DequeIterator )
		{
			return( deck == ((DequeIterator)other).deck
			    &&   cursor == ((DequeIterator)other).cursor );
		}

		return false;
	}
	
// Accessors

	/**
	 * Pointed value.
	 */
	public Object
	get_value()
	{
		return deck.map[cursor];
	}

// Commands

	/**
	 * Rewind to first cell.
	 */
	public void
	at_begin()
	{
		cursor = deck.beg;
	}

	/**
	 * Go past-the-end.
	 */
	public void
	at_end()
	{
		cursor = deck.pte;
	}

	/**
	 * To next cell.
	 */
	public void
	next()
	{
		if( cursor == deck.pte )
		{
			return;		// do not advance any more if at end
		}
			
		++cursor;

		if( cursor == deck.cap )
		{
			cursor = 0;
		}
	}

	/**
	 * To prev cell.
	 */
	public void
	prev()
	{
		if( cursor == deck.beg )
		{
			return;		// do not go back any more if at the begining
		}
	
		--cursor;

		if( cursor < 0 )
		{
			cursor = deck.cap - 1;
		}
	}
}

// vim:ts=4:ai:
