package book.set;

/**
 * Iterator for the Hash class.
 *
 * @author Antoine Dutot
 * @since 19980701
 * @version 0.1
 */
public class HashIterator
	implements Iterator
{
// Attributes

	/**
	 * Hash table containing node.
	 */
	protected Hash hash;
	
	/**
	 * Current node or null.
	 */
	protected HashNode node;

// Constructors

	/**
	 * New null iterator for hash.
	 */
	public
	HashIterator( Hash hash )
	{
		this.hash = hash;
		this.node = null;
	}

	/**
	 * New iterator for hash pointing on node.
	 */
	public
	HashIterator( Hash hash, HashNode node )
	{
		this.hash = hash;
		this.node = node;
	}

	/**
	 * New copy of other.
	 */
	public
	HashIterator( HashIterator other )
	{
		hash = other.hash;
		node = other.node;
	}

// Accessors

	public boolean
	is_end()
	{
		return( node == null );
	}
	
	/**
	 * Pointing on a node ?
	 *
	 * The end iterator of the hash table is not valid.
	 */
	public boolean
	is_valid()
	{
		return( hash != null && node != null );
	}

	public boolean
	equals( HashIterator other )
	{
		return( hash == other.hash && node == other.node );
	}

	public boolean
	equals( Iterator other )
	{
		if( other instanceof HashIterator )
		{
			return( hash == ((HashIterator)other).hash
			    &&   node == ((HashIterator)other).node );
		}

		return false;
	}

// Accessors

	/**
	 * Current hash table.
	 */
	public Hash
	get_hash()
	{
		return hash;
	}

	/**
	 * Current node or null if at end.
	 */
	public HashNode
	get_node()
	{
		return node;
	}

	public Object
	get_value()
	{
		if( node != null )
		{
			return node.get_value();
		}

		return null;
	}

// Commands

	/**
	 * Advance to the next node.
	 */
	public void
	next()
	{
		if( node != null )
		{
			node = hash.get_next_node( node );
		}
	}

	/**
	 * Directly change the hash and node.
	 */
	protected void
	set_node( Hash hash, HashNode node )
	{
		this.hash = hash;
		this.node = node;
	}
}

// vim:ts=4:ai:
