package book.set;

/**
 * Node class for the Hash table allowing to consider it as a set. The hash
 * will be used to know if a key is part or not of the set.
 *
 * @author Antoine Dutot
 * @since 20010519
 * @version 0.1
 */
public class HashSetNode
	extends HashNode
	implements Cloneable
{
// Attributes

	/**
	 * Key and value for the node.
	 */
	public Object keyval;

// Constructors

	/**
	 * New node with no successor.
	 */
	public
	HashSetNode()
	{
		super();
	}
	
	/**
	 * New node indexed by an containing keyval linked to successor.
	 */
	public
	HashSetNode( Object keyval, Linkable successor )
	{
		super( successor );
		this.keyval = keyval;
	}
	
	/**
	 * New node indexed by and containing keyval with no successor.
	 */
	public
	HashSetNode( Object keyval )
	{
		super();
		this.keyval = keyval;
	}

// Accessors
	
	public HashNode
	new_instance( Object key, Object value ) throws CloneNotSupportedException
	{
		if( key != value )
		{
			throw new RuntimeException(
				"with hashset nodes the key must equal the value\n" +
				"see Hash.add(Object,Object)" );
		}
		
		return new HashSetNode( key );
	}

	public boolean
	key_equals( Object key )
	{
		return( keyval.equals( key ) );
	}

	/**
	 * Hash code for any kind of key.
	 */
	public int
	get_hash_code( Object key )
	{
		return key.hashCode();
	}

	public Object
	get_key()
	{
		return keyval;
	}

	public Object
	get_value()
	{
		return keyval;
	}

// Commands

	/**
	 * Change the key. This also change the value.
	 */
	public void
	set_key( Object key )
	{
		keyval = key;
	}

	/**
	 * Change the value. This also change the key.
	 */
	public void
	set_value( Object value )
	{
		keyval = value;
	}
}

// vim:ts=4:ai:
