package book.set;

/**
 * Singly linked node.
 *
 *	A `Linkable' is the base of most linked nodes. It only defines a reference
 *	on the next node in the list. This reference can be null.
 *
 *	An important property of this class is that all copy operations never copy
 *	the `next' attribute(that is, here, the copy makes nothing, in heirs
 *	however other attributes may be copied in the usual way).
 *
 * @author	Antoine Dutot
 * @since 19991022
 * @version 0.1
 */
public class Linkable
{
// Attributes

	/**
	 * Successor or null.
	 */
	public Linkable next;

// Constructors

	/**
	 * New node pointing on null.
	 */
	public
	Linkable()
	{
		next = null;
	}
	
	/**
	 * New node pointing on successor.
	 */
	public
	Linkable( Linkable successor )
	{
		next = successor;
	}

// Accessors

	/**
	 * Successor or null.
	 */
	public Linkable
	get_next()
	{
		return next;
	}

// Commands

	/**
	 * Change the successor without updating it.
	 */
	public void
	set_next( Linkable successor )
	{
		next = successor;
	}
	
	/**
	 * Insert successor between this and the actual successor.
	 *
	 * successor is updated to point on the current succesor of this.
	 */
	public void
	append( Linkable successor )
	{
		successor.next = this.next;
		this.next = successor;
	}
	
	/**
	 * Remove the node just after this one.
	 *
	 * The actual successor is lost, you must have backed it up. It is not
	 * deleted. It is replaced by the node comming just after it if any.
	 */
	public void
	remove_after()
	{
		this.next = this.next.next;
	}
	
	/**
	 * Swap this with `other'.
	 */
	public void
	swap( Linkable other )
	{
		Linkable node;
		
		node = next;
		next = other.next;
		other.next = node;
	}
}

// vim:ts=4:ai:
