package book.set;

/**
 * Iterator for the single linked list class.
 *
 * @author Antoine Dutot
 * @since 19980727
 * @version 0.1
 */
public class LinkedListIterator
	implements Iterator, Cloneable
{
// Attributes
	
	/**
	 * Current position in the list or null.
	 */
	ObjectLinkable node;

// Constructors

	/**
	 * New iterator pointing on nothing.
	 */
	public
	LinkedListIterator()
	{
		node = null;
	}
	
	/**
	 * New iterator pointing on node.
	 */
	public
	LinkedListIterator( ObjectLinkable node )
	{
		this.node = node;
	}

	/**
	 * New iterator pointing on first node of list.
	 */
	public
	LinkedListIterator( LinkedList list )
	{
		this.node = (ObjectLinkable) list.head.next;
	}

// Accessors

	/**
	 * No successor on wich to iterate?
	 */
	public boolean
	is_end()
	{
		return( this.node == null );
	}

	/**
	 * Pointing on the same element as other?
	 */
	public boolean
	equals( LinkedListIterator other )
	{
		return( node == other.node );
	}

	/**
	 * Pointing on the same element as other?
	 */
	public boolean
	equals( Iterator other )
	{
		if( other instanceof LinkedListIterator )
		{
			return( ((LinkedListIterator)other).node == node );
		}

		return false;
	}

	/**
	 * Pointed at list node or null.
	 */
	public ObjectLinkable
	get_node()
	{
		return node;
	}

	/**
	 * Value pointed at or null.
	 */
	public Object
	get_value()
	{
		return node.value;
	}
	
// Commands

	/**
	 * Copy the value of other(but do not point on the same node as other).
	 */
	public void
	copy( LinkedListIterator other )
	{
		node = other.node;
	}

	/**
	 * Change the pointed at node.
	 */
	public void
	set_node( ObjectLinkable node )
	{
		this.node = node;
	}

	/**
	 * Go to next element in the list.
	 */
	public void
	next()
	{
		node = (ObjectLinkable) node.next;
	}
}

// vim:ts=4:ai:
