package book.swing;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * A JLabel + a JField.
 *
 * @author Antoine Dutot
 * @since 20030423
 */
public class LabelPlusField
	extends JPanel
{
// Attributes

	private static final long serialVersionUID = -9041095074835841897L;

	/**
	 * Label of the field.
	 */
	public JLabel label;

	/**
	 * Field.
	 */
	public JTextField fields[];

// Constructors

	public
	LabelPlusField( String label_text, String text, boolean editable, int label_max_width, int max_height )
	{
		Dimension dim;

		setLayout( new BoxLayout( this, BoxLayout.X_AXIS ) );

		label     = new JLabel( label_text );
		fields    = new JTextField[1];
		fields[0] = new JTextField( text );
		dim       = new Dimension( label_max_width, max_height );

		label.setPreferredSize( dim );
		label.setMaximumSize( dim );
		label.setMinimumSize( dim ); 
		dim = new Dimension( 3000, max_height );
		fields[0].setEditable( editable );
		fields[0].setMaximumSize( dim );

		add( label );
		add( fields[0] );
	}

	public
	LabelPlusField( String label_text, int field_count, String text, boolean editable, int label_max_width, int max_height )
	{
		Dimension dim;

		assert( field_count > 0 );

		setLayout( new BoxLayout( this, BoxLayout.X_AXIS ) );

		label  = new JLabel( label_text );
		fields = new JTextField[field_count];
		dim    = new Dimension( label_max_width, max_height );

		label.setPreferredSize( dim );
		label.setMaximumSize( dim );
		label.setMinimumSize( dim );
		dim = new Dimension( 3000, max_height );
		add( label );

		for( int i=0; i<fields.length; ++i )
		{
			fields[i] = new JTextField( text );
			fields[i].setEditable( editable );
			fields[i].setMaximumSize( dim );
			add( fields[i] );
		}
	}

// Accessors

	public JLabel
	get_label()
	{
		return label;
	}

	public JTextField
	get_text_field()
	{
		return fields[0];
	}

	public int
	get_text_field_count()
	{
		return fields.length;
	}

	public JTextField
	get_text_field( int i )
	{
		return fields[i];
	}

	public String
	get_text()
	{
		return fields[0].getText();
	}

	public String
	get_text( int i )
	{
		return fields[i].getText();
	}

// Commands

	public void
	set_label( String text )
	{
		label.setText( text );
	}

	public void
	set_text( String text )
	{
		fields[0].setText( text );
	}

	public void
	set_text( int i, String text )
	{
		fields[i].setText( text );
	}

	public void
	set_label_font( Font font )
	{
		label.setFont( font );
	}

	public void
	set_fields_font( Font font )
	{
		for( int i=0; i<fields.length; ++i )
			fields[i].setFont( font );
	}

	public void
	set_field_editable( int i, boolean on )
	{
		fields[i].setEditable( on );
	}

	public void
	set_fields_editable( boolean on )
	{
		for( int i=0; i<fields.length; ++i )
			fields[i].setEditable( on );
	}

	public void
	set_field_horizontal_alignment( int i, int alignment )
	{
		fields[i].setHorizontalAlignment( alignment );
	}

	public void
	set_fields_horizontal_alignment( int alignment )
	{
		for( int i=0; i<fields.length; ++i )
			fields[i].setHorizontalAlignment( alignment );
	}

	public void
	set_icon( ImageIcon icon )
	{
		label.setIcon( icon );
	}

	public void
	add_fields_action_listener( ActionListener listener )
	{
		for( int i=0; i<fields.length; ++i )
			fields[i].addActionListener( listener );
	}
}
