package book.swing;

import java.awt.*;
import java.util.*;

import book.geom.*;

/**
 * PlotterLineCurve.
 *
 * @author Antoine Dutot
 * @since 20030523
 */
public class PlotterLineCurve
	implements PlotterCurve
{
// Attributes

	/**
	 * Drawing context.
	 */
	protected SwingContext ctx;

	/**
	 * Plotter 2D.
	 */
	protected Plotter2d plotter;

	/**
	 * Compress points when possible?.
	 */
	protected boolean compress = true;

	/**
	 * Curve name.
	 */
	protected  String name;

	/**
	 * Ordered list of points.
	 */
	protected ArrayList<Point2> points = new ArrayList<Point2>();

	/**
	 * Curve color.
	 */
	protected Color color = Color.BLACK;

// Constructors

	public
	PlotterLineCurve( String name )
	{
		this.name = name;
	}

// Accessors

	public int
	get_point_count()
	{
		return points.size();
	}

	public Point2
	get_point( int i )
	{
		return points.get( i );
	}

	public String
	get_name()
	{
		return name;
	}

	public Color
	get_color()
	{
		return color;
	}

// Commands

	public void
	set_name( String name )
	{
		this.name = name;
	}

	public void
	set_color( Color color )
	{
		this.color = color;
	}

	public void
	add_point( float x, float y )
	{
		int n = points.size();

		if( compress && n > 1 )
		{
			Point2 p0 = points.get( n - 2 );
			Point2 p1 = points.get( n - 1 );
	
			if( p0.y == y && p1.y == y )
			     p1.x = x;
			else points.add( new Point2( x, y ) );
		}
		else
		{
			points.add( new Point2( x, y ) );
		}
	}

	public void
	clear()
	{
		points.clear();
	}

	public void
	paint( Graphics2D g )
	{
		if( plotter != null && ctx != null )
		{
			int window = plotter.get_window();
			int n = points.size();
			int m;

			if( n > 0 )
			{
				Point2 p0, p1;

				if( window > 0 )
				{
					m = n;

					if( window < m )
					{
						m -= window;
						p0 = get_point( m );
						m += 1;
					}
					else
					{
						m  = 1;
						p0 = get_point( 0 );
					}
				}
				else
				{
					m  = 1;
					p0 = get_point( 0 );
				}
	
				g.setColor( get_color() );

				for( int i=m; i<n; ++i )
				{
					p1 = get_point( i );

					g.drawLine( plotter.xx(p0.x), plotter.yy(p0.y), plotter.xx(p1.x), plotter.yy(p1.y) );

					p0 = p1;
				}
			}
		}
	}

// Utilities

	/**
	 * Called by the <code>plotter2d</code> each time the curve is inserted in
	 * it or removed from it.
	 */
	public void
	set_plotter( Plotter2d plotter2d )
	{
		if( plotter2d != null )
		{
			plotter = plotter2d;
			ctx     = plotter2d.get_swing_context();
		}
		else
		{
			plotter = null;
			ctx     = null;
		}
	}
}
