package book.swing;

import java.awt.*;
import java.util.*;

import book.geom.*;

/**
 * PlotterMarkersCurve.
 *
 * @author Antoine Dutot
 * @since 20030523
 */
public class PlotterMarkersCurve
	implements PlotterCurve
{
// Attributes

	/**
	 * Drawing context.
	 */
	protected SwingContext ctx;

	/**
	 * Plotter 2D.
	 */
	protected Plotter2d plotter;

	/**
	 * Curve name.
	 */
	protected String name;

	/**
	 * Ordered list of points.
	 */
	protected ArrayList<Point2> points = new ArrayList<Point2>();

	/**
	 * Curve color.
	 */
	protected Color color = Color.GRAY;

// Constructors

	public
	PlotterMarkersCurve( String name )
	{
		this.name = name;
	}

// Accessors

	public int
	get_point_count()
	{
		return points.size();
	}

	public Point2
	get_point( int i )
	{
		return points.get( i );
	}

	public String
	get_name()
	{
		return name;
	}

	public Color
	get_color()
	{
		return color;
	}

// Commands

	public void
	set_name( String name )
	{
		this.name = name;
	}

	public void
	set_color( Color color )
	{
		this.color = color;
	}

	/**
	 * The point is not added if there is already a marker at position x. Only
	 * the x value is used.
	 */
	public void
	add_point( float x, float y )
	{
		int n = points.size();

		if( n > 0 )
		{
			Point2 p = points.get( n - 1 );

			if( p.x != x )
			{
				points.add( new Point2( x, y ) );
			}
		}
		else
		{
			points.add( new Point2( x, y ) );
		}
	}

	public void
	clear()
	{
		points.clear();
	}

	public void
	paint( Graphics2D g )
	{
		if( plotter != null && ctx != null )
		{
			int window = plotter.get_window();
			int n = points.size();
			int m = 0;

			if( n > 0 )
			{
				Point2 p;

				if( window > 0 )
				{
					m = n;

					if( window < m )
						 m -= window;
					else m  = 0;
				}
	
				g.setColor( get_color() );

				for( int i=n-1; i>=m; --i )
				{
					p = get_point( i );

					if( p.x < plotter.xmin )
						break;

					g.drawLine( plotter.xx(p.x), plotter.yy(plotter.ymin), plotter.xx(p.x), plotter.yy(plotter.ymax) );
				}
			}
		}
	}

// Utilities

	/**
	 * Called by the <code>plotter2d</code> each time the curve is inserted in
	 * it or removed from it.
	 */
	public void
	set_plotter( Plotter2d plotter2d )
	{
		if( plotter2d != null )
		{
			plotter = plotter2d;
			ctx     = plotter2d.get_swing_context();
		}
		else
		{
			plotter = null;
			ctx     = null;
		}
	}
}
