package book.swing;

import java.awt.*;
import java.awt.image.*;
import javax.swing.*;

/**
 * Spinner.
 *
 * <p>A spinner is a small logo that changes when a software is doing a task.
 * The most known spinner is the one of the Mosaic program, then used by
 * Netscape and IE.</p>
 *
 * @author Antoine Dutot
 * @since 20030417
 */
public class Spinner
	extends JPanel
	implements ImageObserver
{
// Attributes

	private static final long serialVersionUID = -5596564303122880867L;

	/**
	 * Images of the animation.
	 */
	protected Image images[];

	/**
	 * Currently displayed image.
	 */
	protected int current = 0;

// Constructors

	public
	Spinner( String base_URL, String ext, int count )
	{
		this( base_URL, ext, count, 0 );
	}

	public
	Spinner( String base_URL, String ext, int count, int begin_at )
	{
		assert( count > 0 );

		StringBuffer buf = new StringBuffer();
		String       url;

		buf.append( base_URL );
		images = new Image[count];

		for( int i=0; i<count; ++i )
		{
			buf.setLength( base_URL.length() );
			buf.append( begin_at+i );
			buf.append( ext );
			
			url       = buf.toString();
			images[i] = SwingContext.load_image( url );

			if( images[i] == null )
				System.err.println( "cannot find ressource '"+url+"'" );
		}

		if( images[0] != null )
		{
			images[0].getWidth( this );
		//	images[0].getHeight( this );
		}
	}

// Accessors

// Commands

	public void
	step()
	{
		current = ( ( current + 1 ) % images.length );
		repaint();
	}

	public void
	paintComponent( Graphics g )
	{
		super.paintComponent( g );

		if( images[current] != null )
		     g.drawImage( images[current], 0, 0, null );
		else g.drawString( "no image", 0, 13 );
	}

// Events

	public boolean
	imageUpdate( Image img, int infoflags, int x, int y, int width, int height )
	{
		if( ( ( infoflags & WIDTH ) > 0 ) && ( ( infoflags & HEIGHT ) > 0 ) )
		{
			Dimension dim;

			dim = new Dimension( width, height );

			setMinimumSize( dim );
			setMaximumSize( dim );
			setPreferredSize( dim );

			return false;
		}
		else
		{
			return true;
		}
	}
}
