/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package org.miv.graphstream.algorithm;

import java.util.ArrayList;
import java.util.Iterator;

import org.miv.graphstream.graph.Graph;
import org.miv.graphstream.graph.Node;

public class GraphOfLife implements DynamicAlgorithm
{
// Attributes
	
	protected Graph graph;

	protected ArrayList<Rule> rules = new ArrayList<Rule>();
	
	public GraphOfLife( Graph graph )
	{
		setGraph( graph );
	}
	
	public void addRule( Rule rule )
	{
		rules.add( rule );
	}
	
	public void begin()
    {
    }

	public void end()
    {
    }

	public void compute()
    {
		Iterator<? extends Node> i = graph.getNodeIterator();
		
		while( i.hasNext() )
		{
			Node n = i.next();
			
			for( Rule rule: rules )
			{
				rule.prepare( n );
			}
		}
		
		i = graph.getNodeIterator();
		
		while( i.hasNext() )
		{
			Node n = i.next();
			
			for( Rule rule: rules )
			{
				rule.apply( n );
			}
		}
    }

	public Graph getGraph()
    {
	    return graph;
    }

	public void setGraph( Graph graph )
    {
		this.graph = graph;
    }
	
// Nested classes
	
	/**
	 * 
	 */
	public interface Rule
	{
		void prepare( Node node );
		
		void apply( Node node );
	}
	
	public class LifeRule implements Rule
	{
		public void apply( Node node )
        {
			if( node.hasAttribute( "LifeJudgment" ) )
			{
				if( node.getAttribute( "LifeJudgment" ).equals( "DEATH" ) )
				{
					node.getGraph().removeNode( node.getId() );
				}
			}
        }

		public void prepare( Node node )
        {
			if( node.getDegree() < 2 || node.getDegree() > 3 )
			{
				node.setAttribute( "LifeJudgment", "DEATH" );
			}
			else
			{
				node.setAttribute( "LifeJudgment", "LIFE" );
			}
        }		
	}

	protected int id = 0;
	
	public class BirthRule implements Rule
	{
		@SuppressWarnings("unchecked")
        public void apply( Node node )
        {
			if( node.hasAttribute( "Birth" ) )
			{
				ArrayList<String> neighbors = (ArrayList<String>) node.getAttribute( "Birth" );
				
				Graph  g = node.getGraph();
				String n = String.format( "%d", id++ ); 
				
				g.addNode( n );
				
				for( String ni: neighbors )
					g.addEdge( n+"-"+ni, n, ni );
			}
        }

		public void prepare( Node node )
        {
			if( node.getDegree() == 3 )
			{
				ArrayList<String> neighbors = new ArrayList<String>();
				Iterator<?extends Node> i = node.getNeighborNodeIterator();
				
				while( i.hasNext() )
				{
					neighbors.add( i.next().getId() );
				}
				
				node.setAttribute( "Birth", neighbors );
			}
        }
	}
}