package org.miv.graphstream.examples.esm08;

import java.util.ArrayList;
import java.util.Iterator;

import org.miv.graphstream.graph.Edge;
import org.miv.graphstream.graph.Graph;
import org.miv.graphstream.graph.Node;

/**
 * A Mobile device.
 */
public class MobileDevice {
	protected double x, y;
	protected double targetx, targety;
	protected double speed;
	protected int pause;
	protected Graph graph;
	protected Node node;
	public MobileDevice( Graph graph, String name ) {
		this.graph = graph;
		this.x = Math.random();
		this.y = Math.random();
		nextTarget();
		node = graph.addNode( name );
	}
	public void next() {
		if( pause > 0 ) {
			pause--;
		} else if( atTarget() ) {
			nextTarget();
		} else {
			move();
		}
	}
	protected boolean closeTo( Node other ) {
		float pos[] = graph.algorithm().getNodePosition( other );
		double dx = pos[0]-x;
		double dy = pos[1]-y;
		return ( Math.sqrt( dx*dx + dy*dy ) < 0.11f );
	}
	protected void nextTarget() {
		targetx = Math.random();
		targety = Math.random();
		speed   = Math.random()*0.05f;
		pause   = (int)(Math.random()*10);
	}
	protected void move() {
		x += (targetx-x)*speed;
		y += (targety-y)*speed;
		node.setAttribute( "x", x );
		node.setAttribute( "y", y );
	}
	protected boolean atTarget() {
		return ( x >= targetx-0.1f
		      && x <= targetx+0.1f
		      && y >= targety-0.1f
		      && y <= targety+0.1f );
	}
	protected void checkEdges() {
		Iterator<?extends Edge> edges = node.getEdgeIterator();
		ArrayList<String> toRemove = new ArrayList<String>();
		while( edges.hasNext() ) {
			Edge edge = edges.next();
			if( ! closeTo( edge.getOpposite( node ) ) ) {
				toRemove.add( edge.getId() );
			}
		}

		for( String edge: toRemove )
			graph.removeEdge( edge );

		Iterator<?extends Node> nodes = graph.getNodeIterator();
		while( nodes.hasNext() ) {
			Node other = nodes.next();
			if( other != node && closeTo( other ) ) {
				if( ! node.hasEdgeToward( other.getId() ) ) {
					graph.addEdge(
						String.format( "%s-%s", node.getId(), other.getId() ),
						node.getId(), other.getId() );
				}
			}
		}
	}
}