package org.miv.graphstream.examples.esm08;

import java.util.ArrayList;

import org.miv.graphstream.graph.Graph;
import org.miv.graphstream.graph.Node;
import org.miv.graphstream.graph.implementations.DefaultGraph;

/**
 * Mobility model.
 */
public class MobilityModel {
	public static void main( String args[] ) {
		new MobilityModel();
	}
	protected ArrayList<MobileDevice> devices = new ArrayList<MobileDevice>();
	protected int deviceCount = 200;
	protected int steps = 10000;
	protected Graph graph;
	public MobilityModel() {
		graph = new DefaultGraph();
		graph.display( false );
		graph.addAttribute( "stylesheet", styleSheet );
		Node n0 = graph.addNode( "N_0" );
		Node n1 = graph.addNode( "N_1" );
		n0.setAttribute( "xy", 0, 0 );
		n1.setAttribute( "xy", 1, 1 );
		addMobileDevices(deviceCount);
		for( int i=0; i<steps; i++ ) {
			runMobileDevices();
			checkEdges();
			sleep();
		}
	}
	protected void addMobileDevices(int number) {
		for( int i=0; i<number; ++i ) {
			devices.add( new MobileDevice( graph, String.format( "%d", i ) ) );
		}
	}
	protected void runMobileDevices() {
		for( MobileDevice device: devices )
			device.next();
	}
	protected void sleep() {
		try { Thread.sleep( 100 ); } catch( InterruptedException e ) {}
	}
	protected void checkEdges() {
		for( MobileDevice device: devices )
			device.checkEdges();
	}
	
	protected static final String styleSheet =
		"node { width: 5px; }" +
		"edge  { color: #404040; }";
}