/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package org.miv.graphstream.tool;

import java.io.IOException;

import org.miv.graphstream.graph.Graph;
import org.miv.graphstream.graph.implementations.DefaultGraph;
import org.miv.graphstream.algorithm.Algorithms;
import org.miv.graphstream.io.GraphParseException;
import org.miv.util.NotFoundException;

/**
 * Output a degree distribution in text form.
 *
 * @author Antoine Dutot
 * @author Yoann Pign
 * @since 2007
 */
public class DegreeDistribution
{
	protected String filename;
	
	public static void main( String args[] )
	{
		new DegreeDistribution( args );
	}
	
	public DegreeDistribution( String args[] )
	{
		for( int i=0; i<args.length; ++i )
		{
			if( args[i].startsWith( "-" ) )
			{
				usage( "unknown option %s.%n", args[i] );
			}
			
			if( filename != null )
				usage( "only one graph file name can be given at once (first=%s, second=%s).%n", filename, args[i] );
			
			filename = args[i];
		}
		
		if( filename == null )
			usage( "needs at least the filename of a graph.%n" );
		
		degreeDistribution();
	}
	
	protected void usage( String msg, Object ... params )
	{
		if( msg != null )
			System.err.printf( msg, params );
		
		System.err.printf( "%nUsage: %s <graph-name>%n", getClass().getName() );
		System.err.printf( "%n" );
		System.exit( 1 );
	}
	
	protected void degreeDistribution()
	{
		Graph graph = new DefaultGraph();
		Algorithms algos = new Algorithms( graph );
		
		try
		{
			graph.read( filename );
		}
		catch( NotFoundException e )
		{
			e.printStackTrace();
			System.exit( 1 );
		}
		catch( IOException e )
		{
			e.printStackTrace();
			System.exit( 1 );
		}
		catch( GraphParseException e )
		{
			e.printStackTrace();
			System.exit( 1 );
		}
		
		int[] dd = algos.getDegreeDistribution();
		
		for( int i=0; i<dd.length; i++ )
		{
			System.out.printf( "%d%n", dd[i] );
		}
		
		System.out.flush();
	}
}