/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package org.miv.graphstream.tool;

import java.io.IOException;

import org.miv.graphstream.algorithm.generator.Generator;
import org.miv.graphstream.graph.Graph;
import org.miv.graphstream.graph.implementations.MultiGraph;

/**
 * A small command that generates a grid of any size.
 * 
 * @author Antoine Dutot
 */
public class GridGenerator
{
	public static void main( String args[] )
	{
		if( args.length < 2 )
		{
			System.err.printf( "Usage: GridGenerator <size> <out> [X]%n" );
			System.err.printf( "       <size> is the grid width.%n" );
			System.err.printf( "       <out> the output file.%n" );
			System.err.printf( "       [X] optionnal, generates a grid with diagonals.%n" );
			System.err.printf( "       [D] optionnal, generates a directed grid.%n" );
			System.err.printf( "       [T] Optionnal, generates a tore.%n" );
			System.exit( 1 );
		}
		
		int width = 0;
		
		try
		{
			width = Integer.parseInt( args[0] );
		}
		catch( NumberFormatException e )
		{
			System.err.printf( "Size must be an integer (%s)%n", args[0] );
			System.exit( 1 );
		}
		
		if( width <= 0 )
		{
			System.err.printf( "Invalid width %d (width >=1)%n", width );
			System.exit( 1 );
		}
		
		String output    = args[1];
		boolean cross    = false;
		boolean directed = false;
		boolean tore     = false;
		
		if( args.length > 2 )
		{
			for( int i=2; i<args.length; ++i )
			{
				if( args[i].equals( "X" ) )
				{
					cross = true;
				}
				else if( args[i].equals( "D" ) )
				{
					directed = true;
				}
				else if( args[i].equals( "T" ) )
				{
					tore = true;
				}
				else
				{
					System.err.printf( "Unknown argument %s (use 'X', 'D' or 'T')%n", args[1] );
					System.exit( 1 );
				}
			}
		}
			
		System.out.printf( "Generating file '%s' with size %d (directed=%b cross=%b tore=%b)%n", output, width, directed, cross, tore );
		Graph graph = new MultiGraph( "Grid" );
		Generator generator = new org.miv.graphstream.algorithm.generator.GridGenerator( cross, tore, true, directed );
		generator.begin( graph );
		for( int i=0; i<width; i++ )
				generator.nextElement();
		generator.end();
		
		try
        {
	        graph.write( output );
        }
        catch( IOException e )
        {
	        e.printStackTrace();
	        System.exit( 1 );
        }
        System.out.printf( "OK!%n" );
	}
}