/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package org.miv.graphstream.tool;

import java.util.Hashtable;
import java.util.Random;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Locale;

import java.awt.Color;

import java.io.IOException;

import org.miv.graphstream.graph.implementations.DefaultGraph;
import org.miv.graphstream.graph.Node;
import org.miv.graphstream.graph.Edge;
import org.miv.graphstream.algorithm.ConnectedComponents;


import org.miv.graphstream.io.GraphParseException;
import org.miv.graphstream.io.GraphReader;
import org.miv.graphstream.io.GraphReaderFactory;
import org.miv.graphstream.io.GraphReaderListenerHelper;
import org.miv.graphstream.io.GraphReaderListener;

import book.NotFoundException;

/**
 * <p align="justify">
 * Cette classe a pour objectif de construire un arbre couvrant de poids minimum en
 * suivant l'algorithme de Prim.
 * 
 * @author Frederic.Guinand@univ-lehavre.fr
 * @since   20090104
 * @version 20090104
 */

public class Prim {

// Attributes
	
    /** 
     * Quelques constantes (valeurs par d&eacute;faut pour les param&egrave;tres).
     */
    final static double DEFAULT_XMAX = 300;
    final static double DEFAULT_YMAX = 300;
    final static int DEFAULT_NB = 100;
    final static double DEFAULT_DIST = 20;
    final static int DEFAULT_POIDS = 100;
    final static boolean DEFAULT_INTERACTIF = true;
    final static boolean DEFAULT_TRACE = false;

    protected DefaultGraph graph;
    
    /**
     * Taille du plan
     */
    protected double xmax;
    protected double ymax;
    protected int nb;
    protected double seuil;
    protected int poidsmax;
    protected boolean interactif;
    protected boolean trace;
    protected String graphName;
    protected Random alea;
    protected int graineAleatoire;

    // Constructeur
    public Prim(String args[]) {

	xmax = DEFAULT_XMAX;
	ymax = DEFAULT_YMAX;
	nb = DEFAULT_NB;
	seuil = DEFAULT_DIST;
	poidsmax = DEFAULT_POIDS;
	interactif = DEFAULT_INTERACTIF;
	trace = DEFAULT_TRACE;

	initialisation(args);
	genererSommets();
	//hitakey("Les sommets sont l !!");
	genererLiens();
	rendreConnexe();
	hitakey("Prt pour dmarrer Prim !!");
	prim();
	terminaison("Procdure termine");
    }
	



    /**
     * M&eacute;thode de d&eacute;termination d'un arbre couvrant de poids minimum.
     */
    public void prim() {

	Node s,t;
	Edge e;
	ArrayList<Node> arbre = new ArrayList<Node>();
	ArrayList<Edge> cocycle = new ArrayList<Edge>();
	// les sommets et les artes choisies sont marques 

	// choix d'un sommet au hasard et initialisation du co-cycle
	s = (Node)(graph.getNodeSet().toArray()[alea.nextInt(graph.getNodeCount())]);
	arbre.add(s);
	s.addAttribute("color",Color.red);
	Iterator<? extends Edge> aretesSortantes = s.getEdgeIterator();
	while(aretesSortantes.hasNext()) {
	    e = aretesSortantes.next();
	    e.addAttribute("label",((Integer)(e.getAttribute("poids"))).toString());
	    e.addAttribute("color",Color.blue);
	    e.addAttribute("width",4);
	    cocycle.add(e);
	}
	hitakey("choix premier sommet : DONE");
	// Prim
	while(arbre.size() < graph.getNodeCount()) {
	    e = choixArete(cocycle);
	    cocycle.remove(e);
	    e.addAttribute("color",Color.red);
	    e.addAttribute("width",4);
	    e.addAttribute("arbre",new Boolean(true));
	    s = e.getSourceNode();
	    t = e.getOpposite(s);
	    if(arbre.contains(s)) {
		arbre.add(t);
		t.addAttribute("color",Color.red);
		aretesSortantes = t.getEdgeIterator();
	    } else {
		arbre.add(s);
		s.addAttribute("color",Color.red);
		aretesSortantes = s.getEdgeIterator();
	    }
	    hitakey("ajout du nouveau sommet et de la nouvelle arte : DONE");
	    // mise  jour du co-cycle
	    while(aretesSortantes.hasNext()) {
		e = aretesSortantes.next();
		e.addAttribute("label",((Integer)(e.getAttribute("poids"))).toString());
		if(!(arbre.contains(e.getNode0())) || (!(arbre.contains(e.getNode1()))))  {
		    e.addAttribute("color",Color.blue);
		    e.addAttribute("width",4);
		    cocycle.add(e);
		}
	    }
	    hitakey("mj co-cycle relatif nouveau sommet : DONE");
	    // on supprime du co-cycle les artes dont les deux extrmits sont dans l'arbre
	    // en deux tapes, marquage puis suppression
	    for(int i=0;i<cocycle.size();i++) {
		e = cocycle.get(i);
		if(arbre.contains(e.getNode0()) && arbre.contains(e.getNode1())) {
		    e.addAttribute("todie",new Boolean(true));
		}	       
	    }
	    Iterator<? extends Edge> aretes = graph.getEdgeIterator();
	    while(aretes.hasNext()) {
		e = aretes.next();
		if(e.hasAttribute("todie")) {
		    cocycle.remove(e);
		    e.removeAttribute("todie");
		    e.addAttribute("color",Color.green);
		    e.addAttribute("width",4);
		}
	    }
	    hitakey("mj co-cycle suppression artes obsoltes : DONE");
	}
	
	// c'est fini, on va un peu augmenter la taille des artes de l'arbre couvrant
	Iterator<? extends Edge> aretes = graph.getEdgeIterator();
	while(aretes.hasNext()) {
	    e = aretes.next();
	    if(e.hasAttribute("arbre")) {
		e.addAttribute("width",5);
	    } else {
		e.addAttribute("width",1);
	    }
	}
    }

    /**
     *
     */
    public Edge choixArete(ArrayList<? extends Edge>cocycle) {
	Edge choix,e;
	int weight,w;
	choix = cocycle.get(0);
	weight = (Integer)(choix.getAttribute("poids"));
	for(int i=1;i<cocycle.size();i++) {
	    e = cocycle.get(i);
	    w = (Integer)(e.getAttribute("poids"));
	    if(w < weight) {
		choix = e;
		weight = w;
	    }
	}
	return choix;
    }

    // ------------------------------------------------------------------
    // --------------- Gnration du graphe -----------------------------
    // ------------------------------------------------------------------

    /** 
     * M&eacute;thode de g&eacute;n&eacute;ration al&eacute;atoire d'un graphe. 
     */

    public void genererSommets() {
	String idfPt;
	for(int i=0; i<nb;i++) {
	    idfPt = new String("p"+i);
	    Node n = graph.addNode(idfPt);
	    n.addAttribute("x",alea.nextDouble()*xmax);
	    n.addAttribute("y",alea.nextDouble()*ymax);
	    //n.addAttribute("label",idfPt);
	}
    }

    /** 
     * M&eacute;thode de g&eacute;n&eacute;ration des liens en fonction des distances des sommets. 
     */

    public void genererLiens() {

	Iterator<? extends Node> lesSommets;
	lesSommets = graph.getNodeIterator();
	Node s;
	while(lesSommets.hasNext()) {
	    s = lesSommets.next();
	    for(Node t : graph.getNodeSet()) {
		if((!s.hasEdgeFrom(t.getId())) && (s.getId() != t.getId())) {
		    if(distance(s,t) < seuil) {
			Edge e = graph.addEdge(new String(s.getId()+"__"+t.getId()),s.getId(),t.getId());
			e.addAttribute("color",Color.black);
			e.addAttribute("width",1);
			e.addAttribute("poids",alea.nextInt(poidsmax));
		    }
		}
	    }
	}
    }


    /**
     *
     */
    public double distance(Node u, Node v) {
	double xu,yu,xv,yv;
	xu = u.getNumber("x");
	yu = u.getNumber("y");
	xv = v.getNumber("x");
	yv = v.getNumber("y");
	return Math.sqrt((xu-xv)*(xu-xv)+(yu-yv)*(yu-yv));
    }

    /**
     * M&eacute;thode qui rend connexe un graphe non connexe
     *
     */
    public void rendreConnexe() {
	ConnectedComponents cc = new ConnectedComponents(graph);
	Node s,t;
	String edgeName,edgeAltName;
	if(cc.getConnectedComponentsCount() == 1) hitakey("\t --------- connexe ds le dpart !!");
	while(cc.getConnectedComponentsCount() > 1) {
	    s = (Node)(graph.getNodeSet().toArray()[alea.nextInt(graph.getNodeCount())]);
	    t = (Node)(graph.getNodeSet().toArray()[alea.nextInt(graph.getNodeCount())]);
	    if((!s.hasEdgeFrom(t.getId())) && (s.getId() != t.getId())) {
		Edge e = graph.addEdge(new String(s.getId()+"_"+t.getId()),s.getId(),t.getId());
		e.addAttribute("color",Color.black);
		e.addAttribute("width",1);
		e.addAttribute("poids",alea.nextInt(poidsmax));
		if(trace) hitakey("one more");
	    }
	}
    }

    // ____________________________________________________________

    /**
     * M&eacute;thode initialisation()<br>
     * Cette m&eacute;thode initialise la variable al&eacute;atoire et cr&eacute;e une 
     * instance de l'objet java.util.Random() qui est utilis&eacute; &agrave; divers 
     * endroits dans le programme.
     * Cette m&eacute;thode instancie &eacute;galement le graphe.
     */
    private void initialisation(String args[]) {
	graph = new DefaultGraph("");
	//new GraphViewer(graph, false);
	graph.display( false );
	graineAleatoire = (int)(Math.random()*Integer.MAX_VALUE);
	alea = new Random();
	if(args.length == 0) {
	    usage();
	} else {
	    if(args.length > 3) {
		xmax = Double.parseDouble(args[0]);
		ymax = Double.parseDouble(args[1]);
		nb = Integer.parseInt(args[2]);
		seuil = Double.parseDouble(args[3]);
	    }
	    if(args.length > 4) {
		poidsmax = Integer.parseInt(args[4]);
	    } 
	    if(args.length > 5) {
		graineAleatoire = Integer.parseInt(args[5]);
	    }
	    if(args.length > 6) {
		interactif = Boolean.parseBoolean(args[6]);
	    }
	    if(args.length > 7) {
		trace = Boolean.parseBoolean(args[7]);
	    }
	} 
	alea.setSeed(graineAleatoire);
    }	

    /**
     * main()
     */
    public static void main(String args[]) {
	Prim tt = new Prim(args);
    }

    /**
     * Cette m&eacute;thode explique comment la classe doit &ecirc;tre utilis&eacute;e.
     */
    public static void usage() {
	System.out.println("java org.miv.graphstream.enseignement.Prim [paramtres]");
	System.out.println("\t\t xmax : abscisse max");
	System.out.println("\t\t ymax : ordonne max");
	System.out.println("\t\t nb : nombre de sommets \n");
	System.out.println("\t\t seuil : distance max pour qu'il y ait un lien entre deux sommets  \n");
	System.out.println("\t\t poidsmax : valeur de pondration maximum pour une arte  \n");
	System.out.println("\t\t graineAleatoire : valeur entire pour servir de graine au gnrateur  \n");
	System.out.println("\t\t interactif : true/false  \n");
	System.out.println("\t\t trace : true/false  \n");
    }


    /**
     * Pause infinie pour que l'utilisateur puisse comprendre comment se comporte l'algorithme.
     */
    public void hitakey(String msg) {
	if(interactif) {
	    try {
		System.out.print("\n"+msg+"\n");
		System.in.read();
	    } catch(IOException ie) {}
	}
    }

    public void terminaison(String msg) {
	try {
	    System.out.print("\n"+msg+"\n");
	    System.in.read();
	    System.exit(0);
	} catch(IOException ie) {}
    }

}
