/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package org.miv.graphstream.tool;

import java.util.Iterator;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

import org.miv.graphstream.graph.Graph;
import org.miv.graphstream.graph.Node;
import org.miv.graphstream.graph.implementations.DefaultGraph;
import org.miv.util.Environment;

/**
 * A small program that takes a graph as input, remove all leaf nodes and output the graph to
 * another file.
 * 
 * @author Antoine Dutot
 */
public class RemoveLeafs
{
	protected JFileChooser fc = new JFileChooser();
	
	public static void main( String args[] )
	{
		new RemoveLeafs( args );
	}
	
	public RemoveLeafs( String args[] )
	{
		Environment env = Environment.getGlobalEnvironment();
		
		env.readCommandLine( args );
		
		String input  = env.getParameter( "input" );
		String output = env.getParameter( "output" );
		
		if( input == null || input.length() == 0 )
			input = askForInput();
		
		if( output == null || output.length() == 0 )
			output = askForOutput();
		
		convert( input, output );
	}
	
	protected void convert( String input, String output )
	{
		Graph graph = new DefaultGraph();
		
		try
		{
			graph.read( input );
			
			Iterator<? extends Node> nodes = graph.getNodeIterator();
			
			while( nodes.hasNext() )
			{
				Node node = nodes.next();
				
				if( node.getDegree() == 1 )
					nodes.remove();
			}
			
			graph.write( output );
		}
		catch( Exception e )
		{
			e.printStackTrace();
			JOptionPane.showMessageDialog( null,
				String.format( "Error : %s", e.getMessage() ),
				"Error", JOptionPane.ERROR_MESSAGE );
		}
	}
	
	protected String askForInput()
	{
		if( fc.showOpenDialog( null ) == JFileChooser.APPROVE_OPTION )
			return fc.getSelectedFile().getAbsolutePath();
		
		JOptionPane.showMessageDialog( null, "Please specify an input graph file.", "Error", JOptionPane.ERROR_MESSAGE );
		System.exit( 1 );
		return null;
	}
	
	protected String askForOutput()
	{
		if( fc.showSaveDialog( null ) == JFileChooser.APPROVE_OPTION )
			return fc.getSelectedFile().getAbsolutePath();
		
		JOptionPane.showMessageDialog( null, "Please specify an output graph file.", "Error", JOptionPane.ERROR_MESSAGE );
		System.exit( 1 );
		return null;
	}
}