/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package org.miv.graphstream.tool.graphed;

import java.awt.BorderLayout;
import java.awt.Dimension;

import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;
import javax.swing.SwingConstants;
import javax.swing.UIManager;

import org.miv.graphstream.ui.swing.SwingGraphViewer;

/**
 * The graphical user interface.
 * 
 * @author Antoine Dutot
 */
public class UI extends JFrame
{
// Attributes
	
	/**
	 * The global context.
	 */
	protected Context ctx;

// Attributes -- UI

	/**
	 * The GraphStream viewer.
	 */
    protected SwingGraphViewer viewer;
    
    protected JPanel rightPanel;
    
 // Attributes -- Attributes
     
     protected JPanel attrsPanel;
	
// Attributes -- Info and Status panel

    protected UIStatus infoBar;
    
 // Attributes -- Lists
     
     protected UILists lists;
    
// Attributes -- Search
    
    protected JPanel searchPanel;
    
    protected JTextField searchField;
    
    protected JButton clearSearchField;

// Attributes -- Menus

    protected JMenuBar menuBar;
    
    protected JMenu fileMenu;
    
    protected JMenuItem openMenuItem, saveMenuItem;
    
// Attributes -- Modes
    
    protected JToolBar toolBar;

    protected ButtonGroup modesGroup;

	protected JToggleButton mANButton, mAEButton, mSNButton, mSEButton;
    
// Constructors

	/**
	 * New user interface.
	 * @param context The global context.
	 */
	public UI( Context context )
	{
		this.ctx = context;
		
		setLAF();
        
        ImageIcon icon = createImageIcon( "GraphStreamSmallLogo24.png", "" );
        
		viewer = new SwingGraphViewer( ctx.graph, null, true );
		viewer.setQuality( 4 );
		
		viewer.addGraphViewerListener( ctx.actions );
		
		buildMenuAndToolBars();
		buildSearchPanel();
		buildListsPanel();
		buildAttributePanel();
		buildInfoPanel();
		
		add( (JComponent)viewer.getComponent(), BorderLayout.CENTER );
		addWindowListener( ctx.actions );
		
		setSize( 800, 600 );
		setTitle( "GraphEd : Edit your graphs" );
		setIconImage( icon.getImage() );
		setVisible( true );

	}

// Commands

	protected void buildMenuAndToolBars()
	{
		ImageIcon addNodeIcon = createImageIcon( "AddNodeToolIcon32.png", "" );
		ImageIcon addEdgeIcon = createImageIcon( "AddEdgeToolIcon32.png", "" );
		ImageIcon selNodeIcon = createImageIcon( "SelNodeToolIcon32.png", "" );
		ImageIcon selEdgeIcon = createImageIcon( "SelEdgeToolIcon32.png", "" );
		
		menuBar = new JMenuBar();
		toolBar = new JToolBar();
		
		mANButton = new JToggleButton( ctx.actions.modeAddNodes );
		mAEButton = new JToggleButton( ctx.actions.modeAddEdges );
		mSNButton = new JToggleButton( ctx.actions.modeSelectNodes );
		mSEButton = new JToggleButton( ctx.actions.modeSelectEdges );

		mANButton.setIcon( addNodeIcon );
		mAEButton.setIcon( addEdgeIcon );
		mSNButton.setIcon( selNodeIcon );
		mSEButton.setIcon( selEdgeIcon );
		
		mANButton.setHorizontalTextPosition( SwingConstants.CENTER );
		mANButton.setVerticalTextPosition( SwingConstants.BOTTOM );
		mAEButton.setHorizontalTextPosition( SwingConstants.CENTER );
		mAEButton.setVerticalTextPosition( SwingConstants.BOTTOM );
		mSNButton.setHorizontalTextPosition( SwingConstants.CENTER );
		mSNButton.setVerticalTextPosition( SwingConstants.BOTTOM );
		mSEButton.setHorizontalTextPosition( SwingConstants.CENTER );
		mSEButton.setVerticalTextPosition( SwingConstants.BOTTOM );
		
		mANButton.setBorderPainted( false );
		mAEButton.setBorderPainted( false );
		mSNButton.setBorderPainted( false );
		mSEButton.setBorderPainted( false );
		
		modesGroup = new ButtonGroup();
		modesGroup.add( mANButton );
		modesGroup.add( mAEButton );
		modesGroup.add( mSNButton );
		modesGroup.add( mSEButton );
		
		mSNButton.setSelected( true );
		
		toolBar.add( mANButton );
		toolBar.add( mAEButton );
		toolBar.add( mSNButton );
		toolBar.add( mSEButton );
		
		fileMenu = new JMenu( "File" );
		openMenuItem = new JMenuItem( ctx.actions.openAction );
		saveMenuItem = new JMenuItem( ctx.actions.saveAction );
		
		fileMenu.add( openMenuItem );
		fileMenu.add( saveMenuItem );
		menuBar.add( fileMenu );
		setJMenuBar( menuBar );
		add( toolBar, BorderLayout.NORTH );
	}
	
	protected void buildSearchPanel()
	{
		searchPanel = new JPanel();
		searchField = new JTextField();
		clearSearchField = new JButton( "Clear" );

		searchPanel.setLayout( new BoxLayout( searchPanel, BoxLayout.X_AXIS ) );
		searchField.setPreferredSize( new Dimension( 130, 20 ) );
		searchField.setMaximumSize( new Dimension( 500, 25 ) );
		searchPanel.add( new JLabel( "Search : " ) );
		searchPanel.add( searchField );
		searchPanel.add( clearSearchField );
	}
	
	protected void buildListsPanel()
	{
		lists = new UILists( ctx );
		
		lists.add( searchPanel );
		add( lists, BorderLayout.WEST );
	}
	
	protected void buildAttributePanel()
	{
		attrsPanel = new JPanel();
		
		add( attrsPanel, BorderLayout.EAST );
	}
	
	protected void buildInfoPanel()
	{
		infoBar = new UIStatus( ctx );
		add( infoBar, BorderLayout.SOUTH );
	}
	
// Commands -- Utility
	
	/**
	 * Try to set the look and feel for poor Linux users that have the (HORRIBLE) Metal look and
	 * feel by default.
	 */
	protected void setLAF()
	{
		try
        {
			UIManager.LookAndFeelInfo[] installed = UIManager.getInstalledLookAndFeels();
			
			for( int i=0; i<installed.length; i++ )
			{
				if( installed[i].getName().startsWith( "GTK" ) )
				{
//					System.err.printf( "%s", installed[i].getClassName() );
					UIManager.setLookAndFeel( installed[i].getClassName() );
					i = installed.length;
				}
			}
        }
        catch( Exception e )
        {
	        e.printStackTrace();
        }
	}

	/**
	 * Create an image icon from a path that may be relative to the current file system or a jar.
	 * @param path The image URL (in a jar or file system).
	 * @param description The icon description.
	 * @return The icon or null if not found.
	 */
	public ImageIcon createImageIcon( String path, String description )
	{
	    java.net.URL imgURL = getClass().getResource( path );
	    
	    if (imgURL != null)
	    {
	        return new ImageIcon( imgURL, description );
	    }
	    else
	    {
	    	return new ImageIcon( path, description );
	    }
	}

// Constants
	
    private static final long serialVersionUID = 1L;
}