/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package org.miv.graphstream.tool.graphed;

import java.awt.Dimension;

import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;

import org.miv.graphstream.graph.Edge;
import org.miv.graphstream.graph.Element;
import org.miv.graphstream.graph.Graph;
import org.miv.graphstream.graph.GraphListener;
import org.miv.graphstream.graph.Node;

/**
 * The two node and edge lists as well as some actions.
 * 
 * @author Antoine Dutot
 */
public class UILists extends JPanel implements GraphListener
{
// Constants
	
    private static final long serialVersionUID = 1L;

// Attribute
    
    protected Context ctx;
    
    protected JList nodeList;
    
    protected JList edgeList;
    
    protected DefaultListModel nodeListModel;

    protected DefaultListModel edgeListModel;

    protected JPanel nodeActions;
    
    protected JPanel edgeActions;
    
    protected JButton addNode, delNode;
    
    protected JButton addEdge, delEdge;
    
    protected JTextField nodeIdTemplate;
    
    protected JTextField edgeIdTemplate;

// Constructors
    
    public UILists( Context context )
    {
    	ctx = context;

		setLayout( new BoxLayout( this, BoxLayout.Y_AXIS ) );
    	ctx.graph.addGraphListener( this );
    	
    	nodeListModel = new DefaultListModel();
    	edgeListModel = new DefaultListModel();
    	
		nodeList = new JList( nodeListModel );
		edgeList = new JList( edgeListModel );
		
		JScrollPane sp1 = new JScrollPane( nodeList,
				JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
				JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS );
		JScrollPane sp2 = new JScrollPane( edgeList,
				JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
				JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS );

		nodeList.setSelectionMode( ListSelectionModel.MULTIPLE_INTERVAL_SELECTION );
		edgeList.setSelectionMode( ListSelectionModel.MULTIPLE_INTERVAL_SELECTION );
		nodeList.addListSelectionListener( ctx.actions );
		edgeList.addListSelectionListener( ctx.actions );
		
		nodeIdTemplate = new JTextField( "n%d" );
		edgeIdTemplate = new JTextField( "e%d" );
		addNode = new JButton( ctx.actions.addNodeAction );
		addEdge = new JButton( ctx.actions.addEdgeAction );
		delNode = new JButton( ctx.actions.delNodeAction );
		delEdge = new JButton( ctx.actions.delEdgeAction );
		nodeActions = new JPanel();
		edgeActions = new JPanel();
		
		nodeActions.setLayout( new BoxLayout( nodeActions, BoxLayout.X_AXIS ) );
		edgeActions.setLayout( new BoxLayout( edgeActions, BoxLayout.X_AXIS ) );
		
		ImageIcon addIcon = createImageIcon( "Add16.png", "" );
		ImageIcon delIcon = createImageIcon( "Del16.png", "" );
		
		addNode.setText( null );
		delNode.setText( null );
		addEdge.setText( null );
		delEdge.setText( null );
		addNode.setIcon( addIcon );
		addEdge.setIcon( addIcon );
		delNode.setIcon( delIcon );
		delEdge.setIcon( delIcon );
		
		nodeActions.setMaximumSize( new Dimension( 500, 25 ) );
		edgeActions.setMaximumSize( new Dimension( 500, 25 ) );
		nodeIdTemplate.setPreferredSize( new Dimension( 100, 20 ) );
		edgeIdTemplate.setPreferredSize( new Dimension( 100, 20 ) );
		//nodeActions.setLayout( new BoxLayout( nodeActions, BoxLayout.X_AXIS ) );
		//edgeActions.setLayout( new BoxLayout( edgeActions, BoxLayout.X_AXIS ) );
		
		nodeActions.add( new JLabel( "Nodes : " ) );
		edgeActions.add( new JLabel( "Edges : " ) );
		nodeActions.add( addNode );
		nodeActions.add( delNode );
		nodeActions.add( nodeIdTemplate );
		edgeActions.add( addEdge );
		edgeActions.add( delEdge );
		edgeActions.add( edgeIdTemplate );
		
		add( nodeActions );
		add( sp1 );
		add( edgeActions );
		add( sp2 );
    }
    
// Access
    
// Commands

	public void afterNodeAdd( Graph graph, Node node )
	{
		nodeListModel.add( 0, node.getId() );
	}

	public void afterEdgeAdd( Graph graph, Edge edge )
	{
		edgeListModel.add( 0, edge.getId() );
	}

	public void beforeNodeRemove( Graph graph, Node node )
	{
		nodeListModel.removeElement( node.getId() );
	}

	public void beforeEdgeRemove( Graph graph, Edge edge )
	{
		edgeListModel.removeElement( edge.getId() );
	}

	public void attributeChanged( Element element, String attribute, Object oldValue, Object newValue )
	{
	}

	public void beforeGraphClear( Graph graph )
	{
	}
	
	protected void updateStatus()
	{
	}

// Commands

	public ImageIcon createImageIcon( String path, String description )
	{
	    java.net.URL imgURL = getClass().getResource( path );
	    
	    if (imgURL != null)
	    {
	        return new ImageIcon( imgURL, description );
	    }
	    else
	    {
	    	return new ImageIcon( path, description );
	    }
	}

	public void stepBegins(Graph graph, double time)
	{
		// TODO Auto-generated method stub
		
	}
}