/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package org.miv.graphstream.tool.graphed;

import java.awt.Color;
import java.awt.FlowLayout;

import javax.swing.JLabel;
import javax.swing.JPanel;

import org.miv.graphstream.graph.Edge;
import org.miv.graphstream.graph.Element;
import org.miv.graphstream.graph.Graph;
import org.miv.graphstream.graph.GraphListener;
import org.miv.graphstream.graph.Node;

/**
 * The status bar.
 * 
 * @author Antoine Dutot
 */
public class UIStatus extends JPanel implements GraphListener, GraphEdListener
{
// Attributes

    private static final long serialVersionUID = 1L;

    protected Context ctx;
    
    protected JLabel modeField;
    
    protected JLabel nodeCount;
    
    protected JLabel edgeCount;

// Constructors
	
    protected UIStatus( Context context )
    {
		setLayout( new FlowLayout( FlowLayout.LEFT ) );

		ctx         = context;
		nodeCount   = new JLabel( "0 nodes" );
		edgeCount   = new JLabel( "0 edges" );
		modeField   = new JLabel( "Node Sel." );
		
		add( modeField );
		add( nodeCount );
		add( edgeCount );
		
		modeField.setForeground( Color.RED );
		
		ctx.graph.addGraphListener( this );
    }
    
// Commands
	
	protected void update()
	{
		nodeCount.setText( String.format( "%d node%s", ctx.graph.getNodeCount(), ctx.graph.getNodeCount() == 1 ? "" : "s" ) );
		edgeCount.setText( String.format( "%d edge%s", ctx.graph.getEdgeCount(), ctx.graph.getEdgeCount() == 1 ? "" : "s" ) );
	}
	
	protected void updateMode()
	{
		switch( ctx.getMode() )
		{
			case ADD_NODES:
				modeField.setText( "Node Add" );
				break;
			case ADD_EDGES:
				modeField.setText( "Edge Add" );
				break;
			case SEL_NODES:
				modeField.setText( "Node Sel." );
				break;
			case SEL_EDGES:
				modeField.setText( "Edge Sel." );
				break;
		}
	}

// GraphListener
	
	public void afterEdgeAdd( Graph graph, Edge edge )
    {
		update();
    }

	public void afterNodeAdd( Graph graph, Node node )
    {
		update();
    }

	public void attributeChanged( Element element, String attribute, Object oldValue,
            Object newValue )
    {
		update();
    }

	public void beforeEdgeRemove( Graph graph, Edge edge )
    {
		update();
    }

	public void beforeGraphClear( Graph graph )
    {
		update();
    }

	public void beforeNodeRemove( Graph graph, Node node )
    {
		update();
    }

	public void modeChanged()
    {
		updateMode();
    }

	public void stepBegins(Graph graph, double time)
	{
		// TODO Auto-generated method stub
		
	}
}
