/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package org.miv.graphstream.ui2.swing;

import java.awt.event.KeyEvent;

import org.miv.util.geom.Point3;

/**
 * Utility to centralise the shortcuts and actions for all renderer instances.
 * 
 * @author Antoine Dutot
 */
public class ShortcutManager
{
// Attributes
	
	/**
	 * The viewer to control.
	 */
	protected SwingGraphViewer viewer;
	
	protected float viewPercent = 1;
	
	protected Point3 viewPos = new Point3();
	
	protected float rotation = 0;
	
// Construction
	
	/**
	 * New manager operating on the given viewer.
	 * @param viewer The graph viewer to control.
	 */
	public ShortcutManager( SwingGraphViewer viewer )
	{
		this.viewer = viewer;
	}

// Events
	
	/**
	 * A key has been pressed.
	 * @param event The event that generated the key.
	 */
	public void keyPressed( KeyEvent event )
	{
		if( event.getKeyCode() == KeyEvent.VK_PAGE_UP )
		{
			viewer.setViewPercent( viewer.getViewPercent()-0.05f );
		}
		else if( event.getKeyCode() == KeyEvent.VK_PAGE_DOWN )
		{
			viewer.setViewPercent( viewer.getViewPercent()+0.05f );
		}
		else if( event.getKeyCode() == KeyEvent.VK_LEFT )
		{
			if( ( event.getModifiers() & KeyEvent.ALT_MASK ) != 0 )
			{
				float r = viewer.getViewRotation();
				viewer.setViewRotation( r-5, 0 );				
			}
			else
			{
				float delta = viewer.getRenderer().getGraphMetrics().getDiagonal() / 100;
				Point3 p = viewer.getViewCenter();
				viewer.setViewCenter( p.x + delta, p.y, 0 );
			}
		}
		else if( event.getKeyCode() == KeyEvent.VK_RIGHT )
		{
			if( ( event.getModifiers() & KeyEvent.ALT_MASK ) != 0 )
			{
				float r = viewer.getViewRotation();
				viewer.setViewRotation( r+5, 0 );
			}
			else
			{
				float delta = viewer.getRenderer().getGraphMetrics().getDiagonal() / 100;
				Point3 p = viewer.getViewCenter();
				viewer.setViewCenter( p.x - delta, p.y, 0 );
			}
		}
		else if( event.getKeyCode() == KeyEvent.VK_UP )
		{
			float delta = viewer.getRenderer().getGraphMetrics().getDiagonal() / 100;
			Point3 p = viewer.getViewCenter();
			viewer.setViewCenter( p.x, p.y + delta, 0 );						
		}
		else if( event.getKeyCode() == KeyEvent.VK_DOWN )
		{
			float delta = viewer.getRenderer().getGraphMetrics().getDiagonal() / 100;
			Point3 p = viewer.getViewCenter();
			viewer.setViewCenter( p.x, p.y - delta, 0 );									
		}
	}
	
	/**
	 * A key has been pressed.
	 * @param event The event that generated the key.
	 */
	public void keyReleased( KeyEvent event )
	{
	}
	
	/**
	 * A key has been typed.
	 * @param event The event that generated the key.
	 */
	public void keyTyped( KeyEvent event )
	{
		if( event.getKeyChar() == 'R' )
		{
			viewer.resetView();
		}
		else if( event.getKeyChar() == 'B' )
		{
			viewer.setModeFPS( ! viewer.getModeFPS() );
		}
	}
}