/* (C) Copyright 2009-2013 CNRS (Centre National de la Recherche Scientifique).

Licensed to the CNRS under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The CNRS licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License.

*/

/* Contributors:

Luc Hogie (CNRS, I3S laboratory, University of Nice-Sophia Antipolis) 

*/
 
 
package oscilloscup.data.rendering;


import java.awt.*;

import oscilloscup.data.DataElement;
import oscilloscup.system.*;

/**
 * @author Luc Hogie
 */
public abstract class DataElementRenderer
{
	private Color color = new Color( 0, 0, 255);
	private Stroke stroke = new BasicStroke();


	public final void draw( DataElement object, Space space )
	{
		//System.out.println(getClass().getName() + " is drawing");
		//if ( getClass() == PointAsDotRenderer.class)new Error().printStackTrace();
		Graphics2D graphics = space.getFigureGraphics();
		graphics.setColor(getColor());
		graphics.setStroke(getStroke());
		drawImpl(object, space);
	}


	protected abstract void drawImpl( DataElement object, Space space );

	/**
	 * Returns the color.
	 * @return Color
	 */
	public Color getColor()
	{
		return color;
	}

	/**
	 * Sets the color.
	 * @param color The color to set
	 */
	public void setColor(Color color)
	{
		this.color = color;
	}


	public Stroke getStroke()
	{
	    return stroke;
	}
	
	public void setStroke(Stroke stroke)
	{
	    this.stroke = stroke;
	}
    
    public String getPublicName()
    {
        String s = getClass().getName();
        return s.substring(s.lastIndexOf('.'));
    }
}
