/* (C) Copyright 2009-2013 CNRS (Centre National de la Recherche Scientifique).

Licensed to the CNRS under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The CNRS licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License.

*/

/* Contributors:

Luc Hogie (CNRS, I3S laboratory, University of Nice-Sophia Antipolis) 

*/
 
 
package oscilloscup.data.rendering.figure;



import oscilloscup.data.DataElement;
import oscilloscup.data.Figure;
import oscilloscup.data.Point;
import oscilloscup.system.Dimension;
import oscilloscup.system.Space;
/**
 * @author Luc Hogie
 * 
 * Draws a line between each point of the figure to the barycenter of the set of points in the figure.
 */


public class BarycenterFigureRenderer extends FigureRenderer
{
	/**
	 * @see org.lucci.plt.data.DataRenderer#draw(DataObject, Space)
	 */
	public void drawImpl(DataElement object, Space space)
	{
		Figure pointList = (Figure) object;
		int pointCount = pointList.getPointCount();

		if ( pointCount > 0 )
		{
			double x = pointList.getPointAt( 0 ).getX();
			double y = pointList.getPointAt( 0 ).getY();
	
			for (int i = 1; i < pointCount; ++i)
			{
				x += pointList.getPointAt( i ).getX();
				y += pointList.getPointAt( i ).getY();
			}

			Dimension xDimension = space.getXDimension();
			Dimension yDimension = space.getYDimension();

			int bx = xDimension.convertToGraphicsCoordonateSystem(x / pointCount);
			int by = yDimension.convertToGraphicsCoordonateSystem(y / pointCount);

			for (int i = 0; i < pointCount; ++i)
			{
				Point point = pointList.getPointAt(i);
				int px = xDimension.convertToGraphicsCoordonateSystem(point.getX());
				int py = yDimension.convertToGraphicsCoordonateSystem(point.getY());
				space.getFigureGraphics().drawLine( bx, by, px, py );
			}
		}
	}
    
    public String getPublicName()
    {
        return "Barycenter";
    }
}
