/* (C) Copyright 2009-2013 CNRS (Centre National de la Recherche Scientifique).

Licensed to the CNRS under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The CNRS licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License.

*/

/* Contributors:

Luc Hogie (CNRS, I3S laboratory, University of Nice-Sophia Antipolis) 

*/
 
 
package oscilloscup.data.rendering.figure;




import oscilloscup.data.DataElement;
import oscilloscup.data.Figure;
import oscilloscup.data.Point;
import oscilloscup.system.Dimension;
import oscilloscup.system.Space;
/**
 * @author Luc Hogie
 * 
 * This renderer draws a line between each point and its subsequent point. This is a very
 * common process used to draw curves.
 */


public class ConnectedLineFigureRenderer extends FigureRenderer
{
	/**
	 * @see org.lucci.plt.data.DataRenderer#draw(DataObject, Space)
	 */
	public void drawImpl(DataElement object, Space space)
	{
		Figure pointList = (Figure) object;
		int pointCount = pointList.getPointCount();
		int step = 1;
	
		if (pointCount > 200 && isShownPointReductionEnabled())
		{
			step = (int) (pointCount / 200);
		}

		for (int i = step; i < pointCount; i += step)
		{
			Point point = pointList.getPointAt(i);
			Dimension xDimension = space.getXDimension();
			Dimension yDimension = space.getYDimension();

			int px = xDimension.convertToGraphicsCoordonateSystem(point.getX());
			int py = yDimension.convertToGraphicsCoordonateSystem(point.getY());

			Point previousPoint = pointList.getPointAt( i - step );
			int ppx = xDimension.convertToGraphicsCoordonateSystem(previousPoint.getX());
			int ppy = yDimension.convertToGraphicsCoordonateSystem(previousPoint.getY());

			space.getFigureGraphics().setColor(getColor());
            space.getFigureGraphics().setStroke(getStroke());
			space.getFigureGraphics().drawLine( ppx, ppy, px, py );
		}
	}
    
    public String getPublicName()
    {
        return "connected line";
    }

}
