/* (C) Copyright 2009-2013 CNRS (Centre National de la Recherche Scientifique).

Licensed to the CNRS under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The CNRS licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License.

*/

/* Contributors:

Luc Hogie (CNRS, I3S laboratory, University of Nice-Sophia Antipolis) 

*/
 
 
package oscilloscup.data.rendering.figure;


import java.awt.Polygon;

import oscilloscup.data.*;
import oscilloscup.system.*;
/**
 * @author Luc Hogie
 */
public class PolygonFigureRenderer extends FigureRenderer
{
	private boolean filled = true;

	/**
	 * @see org.lucci.plt.data.DataRenderer#draw(DataObject, Space)
	 */
	public void drawImpl(DataElement object, Space space)
	{
		Figure pointList = (Figure) object;
		int pointCount = pointList.getPointCount();
		Polygon polygon = new Polygon();
	
		for (int i = 0; i < pointCount; ++i)
		{
			Point point = pointList.getPointAt(i);
			int px = space.getXDimension().convertToGraphicsCoordonateSystem(point.getX());
			int py = space.getYDimension().convertToGraphicsCoordonateSystem(point.getY());
			polygon.addPoint(px, py);
		}
		
		if ( filled )
		{
			space.getFigureGraphics().fillPolygon(polygon);
		}
		else
		{
			space.getFigureGraphics().drawPolygon(polygon);
		}
	}

	/**
	 * Returns the filled.
	 * @return boolean
	 */
	public boolean isFilled()
	{
		return filled;
	}

	/**
	 * Sets the filled.
	 * @param filled The filled to set
	 */
	public void setFilled(boolean filled)
	{
		this.filled = filled;
	}
    
    public String getPublicName()
    {
        return "polygone";
    }

}
