/* (C) Copyright 2009-2013 CNRS (Centre National de la Recherche Scientifique).

Licensed to the CNRS under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The CNRS licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License.

*/

/* Contributors:

Luc Hogie (CNRS, I3S laboratory, University of Nice-Sophia Antipolis) 

*/
 
 
package oscilloscup.data.rendering.point;



import java.awt.Color;

import oscilloscup.data.*;
import oscilloscup.system.*;

/**
 * @author Luc Hogie
 */

public class HistogramPointRenderer extends PointRenderer
{
    private double barWidth = 1;

    private Color fillColor = Color.white;

    /**
     * @see org.lucci.plt.data.DataRenderer#draw(DataObject, Space)
     */
    public void drawImpl(DataElement object, Space space)
    {
        Point point = (Point) object;
        Dimension xDimension = space.getXDimension();
        Dimension yDimension = space.getYDimension();

        int x1 = xDimension.convertToGraphicsCoordonateSystem(point.getX() - getBarWidth() / 2);
        int y1 = yDimension.convertToGraphicsCoordonateSystem(point.getY());

        int x2 = xDimension.convertToGraphicsCoordonateSystem(point.getX() + getBarWidth() / 2);
        int y2 = yDimension.convertToGraphicsCoordonateSystem(0);
        space.getFigureGraphics().setColor(this.fillColor);
        space.getFigureGraphics().fillRect(x1, y1, x2 - x1, y2 - y1);
        space.getFigureGraphics().setColor(java.awt.Color.black);
        space.getFigureGraphics().drawRect(x1, y1, x2 - x1, y2 - y1);
    }

    public double getBarWidth()
    {
        return barWidth;
    }

    public void setBarWidth(double barWidth)
    {
        this.barWidth = barWidth;
    }

    public Color getFillColor()
    {
        return fillColor;
    }

    public void setFillColor(Color fillColor)
    {
        this.fillColor = fillColor;
    }
    
    public String getPublicName()
    {
        return "bar chart";
    }

}