/* (C) Copyright 2009-2013 CNRS (Centre National de la Recherche Scientifique).

Licensed to the CNRS under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The CNRS licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License.

*/

/* Contributors:

Luc Hogie (CNRS, I3S laboratory, University of Nice-Sophia Antipolis) 

*/
 
 
package oscilloscup.data.rendering.point;

import java.awt.Font;
import java.awt.font.FontRenderContext;
import java.awt.font.GlyphVector;
import java.awt.geom.Rectangle2D;

import oscilloscup.data.DataElement;
import oscilloscup.data.Point;
import oscilloscup.system.Dimension;
import oscilloscup.system.Space;



/**
 * @author Luc Hogie
 */
public class TextPointRenderer extends PointRenderer
{
	private String text = "";
	private Font font = new Font(null, Font.PLAIN, 12);
	
	/**
	 * @see org.lucci.plt.data.DataRenderer#draw(DataObject, Space)
	 */
	public void drawImpl(DataElement object, Space space)
	{
		Dimension xDimension = space.getXDimension();
		Dimension yDimension = space.getYDimension();

		Point point = (Point) object;
		int x = xDimension.convertToGraphicsCoordonateSystem(point.getX());
		int y = yDimension.convertToGraphicsCoordonateSystem(point.getY());
		space.getFigureGraphics().setColor(getColor());
		FontRenderContext frc = space.getFigureGraphics().getFontRenderContext();
		GlyphVector gv = font.createGlyphVector( frc, text );
		Rectangle2D r = gv.getLogicalBounds();
		x =  x - (int) r.getWidth() / 2 + getXShift();
		y =  y + (int) r.getHeight() / 2 + getYShift();
		space.getFigureGraphics().drawGlyphVector( gv, x, y );
	}

	/**
	 * Returns the font.
	 * @return Font
	 */
	public Font getFont()
	{
		return font;
	}

	/**
	 * Returns the text.
	 * @return String
	 */
	public String getText()
	{
		return text;
	}


	/**
	 * Sets the font.
	 * @param font The font to set
	 */
	public void setFont(Font font)
	{
		if ( font == null )
			throw new IllegalArgumentException( "font cannot be set to null" );

		this.font = font;
	}

	/**
	 * Sets the text.
	 * @param text The text to set
	 */
	public void setText(String text)
	{
		if ( text == null )
		{
			text = null;
		}

		this.text = text;
	}
    
    public String getPublicName()
    {
        return "text";
    }

}
