/* (C) Copyright 2009-2013 CNRS (Centre National de la Recherche Scientifique).

Licensed to the CNRS under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The CNRS licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License.

*/

/* Contributors:

Luc Hogie (CNRS, I3S laboratory, University of Nice-Sophia Antipolis) 
Aurelien Lancin (Coati research team, Inria)
Christian Glacet (LaBRi, Bordeaux)
David Coudert (Coati research team, Inria)
Fabien Crequis (Coati research team, Inria)
Grégory Morel (Coati research team, Inria)
Issam Tahiri (Coati research team, Inria)
Julien Fighiera (Aoste research team, Inria)
Laurent Viennot (Gang research-team, Inria)
Michel Syska (I3S, Université Cote D'Azur)
Nathann Cohen (LRI, Saclay) 
Julien Deantoin (I3S, Université Cote D'Azur, Saclay) 

*/
 
 package toools.thread;

import java.util.Iterator;

/**
 * A generator represents a function than computes and deliver objects in
 * another thread. It attempts bring to Java the functionality of the yield()
 * function in Python. A typical use case for this is the iteration over large
 * data-structures.
 * 
 * @author lhogie
 *
 * @param <T>
 */

public abstract class Generator<T> extends Producer<T>
{
	private T currentValue;
	boolean letProducerWork = false;
	boolean isNextComputed = false;


	@Override
	public final void deliver(T newValue)
	{
		currentValue = newValue;
		letProducerWork = false;

		if ( ! isTerminaisonValue(newValue))
		{
			while ( ! letProducerWork)
			{
				Thread.yield();
			}
		}
	}

	@Override
	public final Iterator<T> iterator()
	{
		internalThread.start();
		
		return new Iterator<T>()
		{
			@Override
			public boolean hasNext()
			{
				if ( ! isNextComputed)
				{
					letProducerWork = true;

					// wait until the producer has computed next value
					while (letProducerWork)
					{
						Thread.yield();
					}

					isNextComputed = true;
				}

				return ! isTerminaisonValue(currentValue);
			}

			@Override
			public T next()
			{
				if ( ! isNextComputed)
				{
					hasNext();
				}

				isNextComputed = false;
				return currentValue;
			}
		};
	}
}