/* (C) Copyright 2009-2013 CNRS (Centre National de la Recherche Scientifique).

Licensed to the CNRS under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The CNRS licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License.

*/

/* Contributors:

Luc Hogie (CNRS, I3S laboratory, University of Nice-Sophia Antipolis) 
Aurelien Lancin (Coati research team, Inria)
Christian Glacet (LaBRi, Bordeaux)
David Coudert (Coati research team, Inria)
Fabien Crequis (Coati research team, Inria)
Grégory Morel (Coati research team, Inria)
Issam Tahiri (Coati research team, Inria)
Julien Fighiera (Aoste research team, Inria)
Laurent Viennot (Gang research-team, Inria)
Michel Syska (I3S, Université Cote D'Azur)
Nathann Cohen (LRI, Saclay) 
Julien Deantoin (I3S, Université Cote D'Azur, Saclay) 

*/

package toools.thread;

import java.util.ArrayList;
import java.util.List;

import toools.io.Cout;
import toools.reflect.Clazz;

public class MultiThreadProcessing2
{
	public static int NB_THREADS_TO_USE = Runtime.getRuntime().availableProcessors() * 2;

	private final List<RR> threads = new ArrayList<>();
	final MultiThreadingException errors = new MultiThreadingException();

	public MultiThreadProcessing2(int nbThreads, String threadName, Class<? extends RR> c)
	{
		Cout.debug("parallel processing using " + nbThreads + " threads");

		// first instantiate the threads
		for (int rank = 0; rank < nbThreads; ++rank)
		{
			RR thread = Clazz.makeInstance(c);
			thread.m = this;
			thread.rank = rank;
			thread.setName(threadName + "-" + rank);
			threads.add(thread);
		}

		// then start them later because they need the complete thread list
		for (RR thread : threads)
		{
			thread.start();
		}

		// all threads are finished, join with them to cleanup and collect
		// exceptions
		for (RR thread : threads)
		{
			try
			{
				thread.join();
			}
			catch (InterruptedException e)
			{
				errors.getThreadLocalExceptions().add(e);
				throw errors;
			}
		}

		// if some errors happened in threads
		if ( ! errors.getThreadLocalExceptions().isEmpty())
		{
			throw errors;
		}

	}

	public static void main(String[] args) throws Throwable
	{
		class ThisThread extends RR
		{
			double progress;

			@Override
			protected void runInParallel()
			{
				rank++;
				System.out.println(rank + " => " + progress());
			}

			double progress()
			{
				double p = 0;

				for (Object t : m.threads)
				{
					p += ((ThisThread) t).progress;
				}

				return p;
			}
		}

		new MultiThreadProcessing2(4, "coucou", ThisThread.class);

		System.out.println("it works!");
	}

}
