package toools.thread;

import java.util.List;
import java.util.Random;

import toools.progression.LongProcess;
import toools.text.TextUtilities;
import toools.thread.MultiThreadProcessing.ThreadSpecifics;

public abstract class ParallelIntervalProcessing
{

	public ParallelIntervalProcessing(int intervalSize, int nbThreads, LongProcess lp)
	{

		double segmentSize = intervalSize / (double) nbThreads;

		new MultiThreadProcessing(nbThreads, lp)
		{
			@Override
			protected void runInParallel(ThreadSpecifics s, List<Thread> threads)
					throws Throwable
			{
				int start = (int) (segmentSize * s.rank);
				int end = s.rank == nbThreads - 1 ? intervalSize
						: (int) (segmentSize * (s.rank + 1));

				process(s, start, end);
			}
		};
	}

	protected abstract void process(ThreadSpecifics s, int lowerBound, int upperBound)
			throws Throwable;

	public static void main(String[] args)
	{
		byte[] b = TextUtilities.pickRandomString(new Random(), 10, 10).getBytes();

		new ParallelIntervalProcessing(b.length, MultiThreadProcessing.NB_THREADS_TO_USE,
				null)
		{

			@Override
			protected void process(ThreadSpecifics s, int lowerBound, int upperBound)
			{
				System.out.println(s.rank + ": " + lowerBound + " =>" + upperBound);

			}

		};
	}

}
